/**CHeaderFile*****************************************************************

  FileName    [cuplus.h]

  PackageName [cuplus] 

  Synopsis    [Extensions of the CU package]

  Description []

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
  Torino, Italy. 
  The  Politecnico di Torino makes no warranty about the suitability of 
  this software for any purpose.  
  It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#ifndef _CUPLUS
#define _CUPLUS

/*---------------------------------------------------------------------------*/
/* Nested includes                                                           */
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include "cudd.h"
#include "st.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Structure & type declarations                                             */
/*---------------------------------------------------------------------------*/

/* only accessed by pointer */
typedef struct profile_info_summary cuplus_profile_info_t; 

/**Enum************************************************************************

  Synopsis    [Type for selecting pruning heuristics.]

  Description []

******************************************************************************/

typedef enum
  {
  Cuplus_ActiveRecursions,
  Cuplus_Recursions,
  Cuplus_RecursionsWithSharing,
  Cuplus_NormSizePrune,
  Cuplus_NormSizePruneLight,
  Cuplus_NormSizePruneHeavy,
  Cuplus_SizePrune,
  Cuplus_SizePruneLight,
  Cuplus_SizePruneHeavy,
  Cuplus_SizePruneLightNoRecur,
  Cuplus_SizePruneHeavyNoRecur,
  Cuplus_None
  }
Cuplus_PruneHeuristic_e;

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/


/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN DdNode * Cuplus_ProfileAndAbstract(DdManager *manager, DdNode *f, DdNode *g, DdNode *cube, cuplus_profile_info_t *profile);
EXTERN DdNode * Cuplus_ProfilePrune(DdManager *manager, DdNode *f, cuplus_profile_info_t *profile, Cuplus_PruneHeuristic_e prune_heuristic, int threshold);
EXTERN cuplus_profile_info_t * Cuplus_ProfileInfoGen(unsigned char dac99_compatible);
EXTERN void Cuplus_ProfileInfoFree(cuplus_profile_info_t *profile);
EXTERN void Cuplus_ProfileInfoPrint(cuplus_profile_info_t *profile);

/**AutomaticEnd***************************************************************/

#endif /* _CUPLUS */

 





