/**CFile***********************************************************************

  FileName    [ddiDdNode.c]

  PackageName [ddi]

  Synopsis    [Functions to access DD nodes]

  Description []

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
  Torino, Italy. 
  The  Politecnico di Torino makes no warranty about the suitability of 
  this software for any purpose.  
  It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#include "ddiInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/
  
/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/
  
/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************
  Synopsis     [Returns the root node of the BDD]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
Ddi_DdNode *
Ddi_DdRootNode (
  Ddi_Dd_t *f
)
{
  Pdtutil_Assert(Ddi_DdReadCode(f)==DDI_MONO_BOOLF, "Monolithic BDD expected");
  return (f->boolf.mono);
}

/**Function********************************************************************
  Synopsis     [Returns variable index of f ]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
int
Ddi_DdNodeReadIndex(
  Ddi_DdNode *f
)
{
  return ((int)Cudd_NodeReadIndex(Ddi_DdNodeToCU(f)));
}

/**Function********************************************************************
  Synopsis     [Returns <em>Then</em> branch of f (complemented if f compl.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
Ddi_DdNode *
Ddi_DdNodeThen(
  Ddi_DdNode *f
)
{
  if (Cudd_IsComplement(Ddi_DdNodeToCU(f))) {
    return (Ddi_DdNodeFromCU(Cudd_Not(Cudd_T(Ddi_DdNodeToCU(f)))));
  }
  else {
    return (Ddi_DdNodeFromCU(Cudd_T(Ddi_DdNodeToCU(f))));
  }
}

/**Function********************************************************************
  Synopsis     [Returns <em>Else</em> branch of f (complemented if f compl.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
Ddi_DdNode *
Ddi_DdNodeElse(
  Ddi_DdNode *f
)
{
  if (Cudd_IsComplement(Ddi_DdNodeToCU(f))) {
    return (Ddi_DdNodeFromCU(Cudd_Not(Cudd_E(Ddi_DdNodeToCU(f)))));
  }
  else {
    return (Ddi_DdNodeFromCU(Cudd_E(Ddi_DdNodeToCU(f))));
  }
}


/**Function********************************************************************
  Synopsis    [Returns true if node is visited]
  Description [Returns true if node is visited]
  SideEffects [None]
  SeeAlso     []
******************************************************************************/
int
Ddi_DdNodeIsVisited (
DdNode *f )
{
  return Cudd_IsComplement(Cudd_Regular(Ddi_DdNodeToCU(f))->next);
}    /* test if node is visited. */


/**Function********************************************************************
  Synopsis    [Marks a node as visited]
  Description [Marks a node as visited]
  SideEffects [None]
  SeeAlso     []
******************************************************************************/
void
Ddi_DdNodeSetVisited(
DdNode *f )
{
  DdNode *fCu = Cudd_Regular(Ddi_DdNodeToCU(f));
  fCu->next = Cudd_Complement(fCu->next);
}


/**Function********************************************************************
  Synopsis    [Clears visited flag]
  Description []
  SideEffects [None]
  SeeAlso     []
******************************************************************************/
void
Ddi_DdNodeClearVisited(
DdNode *f )
{
  DdNode *fCu;
  
  if (Ddi_DdNodeIsVisited(f)) {
    fCu = Cudd_Regular(Ddi_DdNodeToCU(f));
    fCu->next =  Cudd_Regular(fCu->next);
  }
}

/**Function********************************************************************
  Synopsis    [Converts into a Cudd node]
  Description []
  SideEffects [None]
  SeeAlso     []
******************************************************************************/
DdNode *
Ddi_DdNodeToCU(
  Ddi_DdNode *n 
)
{
  return (DdNode *)n;
}

/**Function********************************************************************
  Synopsis    [Converts a Cudd node into a Dd node]
  Description []
  SideEffects [None]
  SeeAlso     []
******************************************************************************/
Ddi_DdNode *
Ddi_DdNodeFromCU(
  DdNode *n
)
{
  return (Ddi_DdNode *)n;
}

