/**CHeaderFile*****************************************************************

  FileName    [ddiInt.h]

  PackageName [ddi]

  Synopsis    [Decision diagram interface for the PdTrav package.]

  Description [External functions and data strucures of the DDI package]

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
    Torino, Italy.
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#ifndef _DDIINT
#define _DDIINT

/*---------------------------------------------------------------------------*/
/* Nested includes                                                           */
/*---------------------------------------------------------------------------*/

#include "ddi.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/
#define DDI_DDARRAY_INIT_SIZE      3  

/* message error */
#define DDI_VARARRAY_OUT_OF_MEM -1
#define DDI_VARARRAY_INDEX_OUT_OF_RANGE -2
#define DDI_DDARRAY_OUT_OF_MEM -3
#define DDI_DDARRAY_INDEX_OUT_OF_RANGE -4
#define DDI_DDARRAY_OR_DDARRAY_FAIL -5
#define DDI_DDUTIL_NOSWAP -6
#define DDI_DDUTIL_NOT_INIT -7

/*---------------------------------------------------------------------------*/
/* Structure declarations                                                     */
/*---------------------------------------------------------------------------*/

/**Struct*********************************************************************
 Synopsis    [Dd Manager]
 Description [The Ddi Dd manager]
******************************************************************************/

struct Ddi_DdMgr {
  char *ddiName;                         /* DDI Structure Name */
  DdManager *mgrCU;                      /* CUDD manager */
  struct Ddi_Dd_t *one;                  /* one constant */
  struct Ddi_Dd_t *zero;                 /* zero constant */
  char **varnames;                       /* names of variables */
  int *varauxids;                        /* auxiliary variable ids */
  int vararraySize;                      /* size of varnames and varauxids */
  int ddNum;                             /* counter of Dd Handles */
  Cudd_ReorderingType autodynMethod;     /* suspended autodyn method */
  int autodynSuspended;                  /* flag active if dynord suspended */

  struct {
    Pdtutil_VerbLevel_e verbosity;
  } settings;
};	

/**Struct*********************************************************************
 Synopsis  [Boolean function]
 Description [Handle to a Boolean function (a BDD)]
******************************************************************************/

struct Ddi_Dd_t {
    int             code;
    union {
      Ddi_DdNode   *mono;          /* ptr to the top node of the function */
      struct Ddi_DdArray_t *part;  /* array of partitions */
    } boolf;

    cuplus_profile_info_t *profileInfo;
    Ddi_DdMgr *mgr;        /* the manager */
};	

/**Struct*********************************************************************
 Synopsis  [Array of boolean functions]
******************************************************************************/

struct Ddi_DdArray_t {
    int num;                             /* number of array elements */
    int nSize;                          /* size of allocated data */
    Ddi_Dd_t **data;                     /* ptr to the array */
};

/**Struct*********************************************************************
 Synopsis  [Array of variables]
******************************************************************************/

struct Ddi_VarArray_t {
  int nSize;                /* Number of array elements allocated */
  Ddi_Var_t **data;          /* Pointer to the array */
};

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/**Enum************************************************************************

  Synopsis    [Type for operation codes.]

  Description [Type for operation codes. Used for selection of DDI
               Boolean operation.]

******************************************************************************/
typedef enum {
  DDI_OP_AND,
  DDI_OP_OR,
  DDI_OP_NOT,
  DDI_OP_EXIST,
  DDI_OP_ANDEXIST,
  DDI_OP_XOR,
  DDI_OP_XNOR,
  DDI_OP_ITE,
  DDI_OP_CONSTRAIN,
  DDI_OP_RESTRICT
} ddi_op_t;

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN Ddi_Dd_t * DdiDdOp(ddi_op_t opcode, Ddi_Dd_t *f, Ddi_Dd_t *g, Ddi_Dd_t *h, Ddi_VarSet_t *vs);
EXTERN void DdiMgrCheckVarArraySize(Ddi_DdMgr *dd);
EXTERN Ddi_Var_t ** DdiVarArrayReadData(Ddi_VarArray_t *array);

/**AutomaticEnd***************************************************************/

#endif /* _DDIINT */
