/**CFile***********************************************************************

  FileName    [ddiMgr.c]

  PackageName [ddi]

  Synopsis    [Functions to deal with DD Managers]

  Description []

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#include "ddiInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/
  
/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/
  
/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis           []

  Description        []

  SideEffects        [To be congruent operationFlag should be a Pdtutil_MgrOp_t
   type, and returnFlag of Pdtutil_MgrRet_t type.]

  SeeAlso            []

******************************************************************************/

int
Ddi_MgrOperation (
  Ddi_DdMgr **ddMgrP                 /* DD Manager Pointer */,
  char *string                       /* String */,
  Pdtutil_MgrOp_t operationFlag      /* Operation Flag */,
  void **voidPointer                 /* Generic Pointer */,
  Pdtutil_MgrRet_t *returnFlagP      /* Type of the Pointer Returned */
  )
{
  char *stringFirst, *stringSecond;

  /*------------ Check for the Correctness of the Command Sequence ----------*/

  if (*ddMgrP == NULL) {
    Pdtutil_Warning (1, "Command Out of Sequence.");
    return (1);
  }

  /*--------------------------- Print All Options ---------------------------*/

  /* Nothing for Now */

  /*----------------------- Take Main and Secondary Name --------------------*/

  Pdtutil_ReadSubstring (string, &stringFirst, &stringSecond);
 
  /*----------------------- Operation on the DD Manager ---------------------*/

  if (stringFirst == NULL) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "DecisionDiagramManager Verbosity %s (%d)\n",
          Pdtutil_VerbosityEnum2String (Ddi_MgrReadVerbosity (*ddMgrP)),
          Ddi_MgrReadVerbosity (*ddMgrP));
        break;
      case Pdtutil_MgrOpStats_c:
        Ddi_MgrPrintStats (*ddMgrP);
        break;
      case Pdtutil_MgrOpMgrRead_c:
        *voidPointer = (void *) *ddMgrP;
        *returnFlagP = Pdtutil_MgrRetDdMgr_c;
        break;
      case Pdtutil_MgrOpMgrSet_c:
        *ddMgrP = (Ddi_DdMgr *) *voidPointer;
        break;
      case Pdtutil_MgrOpMgrDelete_c:
        Ddi_DdiMgrQuit (*ddMgrP);
        *returnFlagP = Pdtutil_MgrOpMgrDelete_c;
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on DDI Manager");
        break;
     }

    return (0);
  }

  /*-------------------------------- Options --------------------------------*/

  if (strcmp(stringFirst, "verbosity")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Ddi_MgrSetVerbosity (*ddMgrP,
          Pdtutil_VerbosityString2Enum (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "Verbosity %s (%d)\n",
          Pdtutil_VerbosityEnum2String (Ddi_MgrReadVerbosity (*ddMgrP)),
          Ddi_MgrReadVerbosity (*ddMgrP));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*------------------------------- No Match --------------------------------*/

  Pdtutil_Warning (1, "Operation on DDI manager not allowed");
  return (1);
}

/**Function********************************************************************
  Synopsis     [Prints on standard outputs statistics on a DD manager]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

void
Ddi_MgrPrintStats (
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  Pdtutil_ChrPrint (stdout, '*', 50);
  fprintf (stdout, "\nDecision Diagram Statistics Summary:\n");

  Cudd_PrintInfo (dd->mgrCU, stdout);

  Pdtutil_ChrPrint (stdout, '*', 50);
  fprintf (stdout, "\n");

  return;
}

/**Function********************************************************************
  Synopsis     [make checks and resizes arrays if required]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
DdiMgrCheckVarArraySize(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  int i;
  int nvars = Ddi_ReadNumVars(dd);

  if (dd->vararraySize<nvars) {
    dd->vararraySize *= 2;
    dd->varnames = Pdtutil_Realloc(char *, dd->varnames, dd->vararraySize);
    dd->varauxids = Pdtutil_Realloc(int, dd->varauxids, dd->vararraySize);
    for (i = dd->vararraySize/2; i<dd->vararraySize; i++) {
      dd->varnames[i] = NULL;
      dd->varauxids[i] = -1;
    }
  }

}

/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/


/**Function********************************************************************
  Synopsis     [Creates a DdManager.]
  SideEffects  [none]
  SeeAlso      [Ddi_DdiMgrQuit]
******************************************************************************/

Ddi_DdMgr *
Ddi_MgrInit (
  char *ddiName                /* Name of the DDI structure */,
  DdManager *CUMgr             /* Input CD manager. Created if NULL */,
  unsigned int nvar            /* Initial Number of Variables */,
  unsigned int numSlots        /* Initial Size of Unique Table */,
  unsigned int cacheSize       /* Initial Size of Computed Table (cache) */,
  unsigned long memorySizeMax  /* Max size of Memory */
  )
{
  Ddi_DdMgr *ddiMgr;
  int i;

  if (memorySizeMax == 0) {
    memorySizeMax = getSoftDataLimit();
#if 0
    fprintf (stdout, "MemoryMax = %d\n", memorySizeMax);
#endif
  }

  /*------------------------- DDI Structure Creation -----------------------*/

  ddiMgr = Pdtutil_Alloc (Ddi_DdMgr, 1);
  if (ddiMgr == NULL) {
    return (NULL);
  }

  /*------------------- Set the Name of the DDI Structure ------------------*/

  if (ddiName != NULL) {
     Ddi_SetDdiName (ddiMgr, ddiName);
  } else {
     Ddi_SetDdiName (ddiMgr, "ddm");
  }

  /*------------------------------- Set Fields -----------------------------*/

  if (CUMgr == NULL) {
    CUMgr = Cudd_Init (nvar, 0, numSlots, cacheSize, memorySizeMax);
    if (CUMgr == NULL) {
      Pdtutil_Free (ddiMgr);
      return (NULL);
    }
  }

  ddiMgr->mgrCU = CUMgr;
  ddiMgr->one = Ddi_DdCreateFromCU(ddiMgr,Cudd_ReadOne(CUMgr));
  ddiMgr->zero = Ddi_DdCreateFromCU(ddiMgr,Cudd_Not(Cudd_ReadOne(CUMgr)));
  ddiMgr->ddNum = 0;
  ddiMgr->autodynMethod = (Cudd_ReorderingType) 0; 
  ddiMgr->autodynSuspended = 0; 

  /*---------------- Init variable names and auxids ------------------------*/

  ddiMgr->varnames = NULL;
  ddiMgr->varauxids = NULL;
  if (nvar > PDUTIL_INITIAL_ARRAY_SIZE) {
    ddiMgr->vararraySize = nvar;
  } else {
    ddiMgr->vararraySize = PDUTIL_INITIAL_ARRAY_SIZE;
  }
  ddiMgr->varnames = Pdtutil_Alloc(char *, ddiMgr->vararraySize);
  for (i=0; i<ddiMgr->vararraySize; i++) {
    ddiMgr->varnames[i] = NULL;
  }

  ddiMgr->varauxids = Pdtutil_Alloc(int, ddiMgr->vararraySize);
  for (i=0; i<ddiMgr->vararraySize; i++) {
    ddiMgr->varauxids[i] = (-1);
  }

  ddiMgr->settings.verbosity = Pdtutil_VerbLevelUsrMax_c;

  return (ddiMgr);
}

/**Function********************************************************************
  Synopsis     [Closes a DdManager.]
  SideEffects  [none]
  SeeAlso      [Ddi_MgrInit]
******************************************************************************/
void
Ddi_DdiMgrQuit(
  Ddi_DdMgr *dd  /* dd manager */
)
{
  Pdtutil_Free(dd->ddiName);
  Ddi_DdFree(dd->one);
  Ddi_DdFree(dd->zero);
  Pdtutil_StrArrayFree(dd->varnames,Ddi_ReadNumVars(dd));
  Pdtutil_Free(dd->varauxids);

  Cudd_Quit(dd->mgrCU);

  Pdtutil_Free(dd);
}


/**Function********************************************************************
  Synopsis     [Creates a copy of a DdManager.]
  SideEffects  [none]
  SeeAlso      [Ddi_DdiMgrQuit]
******************************************************************************/
Ddi_DdMgr *
Ddi_MgrDup (
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  Ddi_DdMgr *dd2;
  char **varnames;
  int *varauxids;
  unsigned int nvar         = Ddi_ReadNumVars(dd); 
  unsigned int numSlots     = DDI_UNIQUE_SLOTS /*Ddi_ReadSlots(dd)*/;
  unsigned int cacheSize    = Ddi_ReadCacheSlots(dd);
  unsigned int maxCacheSize = DDI_MAX_CACHE_SIZE;

  Cudd_ReorderingType dynordMethod;
  int autodyn = Ddi_ReadReorderingStatus (dd, &dynordMethod);  

  dd2 = Ddi_MgrInit (NULL, NULL, nvar, numSlots, cacheSize, maxCacheSize);
  if (dd2==NULL) {
    return (NULL);  
  }

  varnames = Ddi_MgrReadVarnames (dd);
  varauxids = Ddi_MgrReadVarauxids (dd);
  dd2->varnames = Pdtutil_StrArrayDup (varnames, nvar);
  dd2->varauxids = Pdtutil_IntArrayDup (varauxids, nvar);

  Ddi_MgrAlign (dd2, dd);

  if (autodyn) {
    Ddi_MgrAutodynEnable (dd2, dynordMethod);
  }

  if (dd->autodynSuspended) {
    Ddi_MgrAutodynSuspend(dd2);
  }

  return (dd2);

}


/**Function********************************************************************
  Synopsis     [Reorders all DDs in a manager.]
  Description  [Reorders all DDs in a manager according to the input
    order. The input specification may be partial, i.e.
    it may include only a subset of variables.
    ]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

void
Ddi_MgrShuffle (
  Ddi_DdMgr *dd      /* dd Manager to be Alligned */,
  int *sortedIds     /* Array of sorted ids */,
  int nids           /* Number of ids */
)
{
  int *newperm, *ids;
  int nv, i, j, result;

  nv = Ddi_ReadNumVars(dd);

  Pdtutil_Assert (nids<=nv, "Input array for MgrShuffle is too large.");

  newperm = Pdtutil_Alloc(int,nv); 
  ids = Pdtutil_Alloc(int,nv); 

  for(i=0; i<nv; i++) {
    newperm[i] = Ddi_ReadInvPerm(dd,i);
  }

  for(i=0; i<nv; i++) {
    ids[i] = -1;
  }

  for(i=0; i<nids; i++) {
    Pdtutil_Assert(sortedIds[i]<nv,"imput id out of range in MgrShuffle"); 
    ids[sortedIds[i]] = 1;
  }

  for(i=0,j=0; i<nv; i++) {
    /* if variable is in sortedIds shuffle (pick up from sortedIds) */
    if (ids[newperm[i]] > 0) {
      newperm[i] = sortedIds[j++];
    }
  }

  Pdtutil_Assert (j==nids, "Not all input ids used for MgrShuffle.");

  if (Ddi_MgrReadVerbosity (dd) >= Pdtutil_VerbLevelDevMin_c) {
    fprintf(stdout,"Partial Sorted ids: ");
    for(i=0; i<nids; i++) {
      fprintf(stdout,"%d ", sortedIds[i]);
    }
    fprintf(stdout,"\n");
    fprintf(stdout,"Full Sorted ids: ");
    for(i=0; i<nv; i++) {
      fprintf(stdout,"%d ", newperm[i]);
    }
    fprintf(stdout,"\n");
  }

  result = Cudd_ShuffleHeap (Ddi_MgrReadMgrCU (dd), newperm);

  Pdtutil_Warning (result==0, "ERROR during ShuffleHeap.");

  Pdtutil_Free (newperm);
  Pdtutil_Free (ids);

  return;
}


/**Function********************************************************************
  Synopsis     [Aligns the order of two managers.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

void
Ddi_MgrAlign (
  Ddi_DdMgr *dd            /* dd manager to be aligned */,
  Ddi_DdMgr *ddRef         /* reference dd manager */
)
{
  int *newperm;
  int nv, nvRef, i, j, idRef, result;

  nv = Ddi_ReadNumVars(dd);
  nvRef = Ddi_ReadNumVars(ddRef);

  newperm = Pdtutil_Alloc(int,nv); 

  for(i=0,j=0; i<nvRef; i++) {
    idRef = Ddi_ReadInvPerm(ddRef,i);
    if (idRef < nv) { 
      /* variable has a corresponding one in dd */
      newperm[j++] = idRef;
    }
  }

  if (Ddi_MgrReadVerbosity (dd) >= Pdtutil_VerbLevelDevMin_c) {
    fprintf(stdout,"Align ids: ");
    for(i=0; i<nv; i++) {
      fprintf(stdout,"%d ", newperm[i]);
    }
    fprintf(stdout,"\n");
  }

  Pdtutil_Assert(j==nv,"missing variables when aligning managers");

  result = Cudd_ShuffleHeap(Ddi_MgrReadMgrCU(dd),newperm);

  if (!result) {
    fprintf(stderr,"ERROR during ShuffleHeap\n");
  }

  Pdtutil_Free(newperm);

  return;
}


/**Function********************************************************************
  Synopsis    [Create groups of 2 variables ]
  Description [Create groups of 2 variables: variables of corresponding 
               indexes in vfix and vmov are coupled. If vmov[i] and vfix[i]
               are not adjacent in the ordering, vmov[i] is moved to the
               position below vfix[i].
              ]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/

void
Ddi_MgrCreateGroups2(
  Ddi_DdMgr *dd              /* manager */,
  Ddi_VarArray_t *vfix       /* first array */,
  Ddi_VarArray_t *vmov       /* first array */
)
{
  int i, j, fixpos, movpos, nvars, nvarstot;
  int *tmpIds, *sortedIds, *grpIds;
  Ddi_Var_t *fv, *mv;
  int do_shuffle = 0;

  Pdtutil_Assert(Ddi_VarArray_n(vfix)==Ddi_VarArray_n(vmov),
    "Variable arrays with different size in grouping");

  nvars = Ddi_VarArray_n(vfix);
  nvarstot = Ddi_ReadNumVars(dd);

  /* moving vmov */

  tmpIds = Pdtutil_Alloc(int,nvarstot); 
  grpIds = Pdtutil_Alloc(int,nvarstot); 
  sortedIds = Pdtutil_Alloc(int,nvarstot); 

  /* creating interleaved ordering for group variables */

  /* initialize tmpIds with present ordering */
  for (i=0,j=0; i<nvarstot; i++) {
    tmpIds[i] = Ddi_ReadInvPerm(dd,i); 
    grpIds[i] = -1;
  }

  for (i=0,j=0; i<nvars; i++) {
    /* obtain variable positions and ids */
    fv = Ddi_VarArrayFetch(vfix,i);
    mv = Ddi_VarArrayFetch(vmov,i);
    fixpos = Ddi_VarCurrPos(dd,fv);
    movpos = Ddi_VarCurrPos(dd,mv);

    if (abs(fixpos-movpos)!=1) /* check enabling shuffle */
      do_shuffle = 1;

    /* clear tmpIds, leaving variables outside groups */
    Pdtutil_Assert(tmpIds[fixpos]==Ddi_VarIndex(fv),
      "id mismatch creating groups");
    Pdtutil_Assert(tmpIds[movpos]==Ddi_VarIndex(mv),
       "id mismatch creating groups");
    tmpIds[fixpos] = -1;
    tmpIds[movpos] = -1;

    /* collect fix variable array ids */
    grpIds[fixpos] = i;
  }

  if (do_shuffle) {
    /*
     * now group variables may be retrieved from grpIds, non group 
     * variables from tmpIds. Do merging
     */
    for (i=0,j=0; i<nvarstot; i++) {
      if (tmpIds[i] >= 0) {
        /* non group variable: keep as it is in new ordering */
        Pdtutil_Assert(grpIds[i]==-1,
          "a variable appears to be both group and no group");
        sortedIds[j++] = tmpIds[i];
      }
      else if (grpIds[i]>=0) { /* skip old positions of vmov variables */ 
        /* group variable: put a group (2 vars) in final ordering */
        Pdtutil_Assert(grpIds[i]<nvars,
          "invalid array index while grouping variables");
        fv = Ddi_VarArrayFetch(vfix,grpIds[i]);
        mv = Ddi_VarArrayFetch(vmov,grpIds[i]);
        fixpos = Ddi_VarCurrPos(dd,fv);
        movpos = Ddi_VarCurrPos(dd,mv);
	/* keep relative ordering within group */
        if (fixpos<movpos) {
          sortedIds[j++] = Ddi_VarIndex(fv);
          sortedIds[j++] = Ddi_VarIndex(mv);
        }
        else {
          sortedIds[j++] = Ddi_VarIndex(mv);
          sortedIds[j++] = Ddi_VarIndex(fv);
        }
      }
    } /* end for */

    Pdtutil_Assert(j==nvarstot,"sorted id array not completed while grouping");

    /* do variable shuffle */
    Ddi_MgrShuffle (dd, sortedIds, nvarstot);

  }

  for (i=0,j=0; i<nvars; i++) {
    /* obtain variable positions and ids */
    fv = Ddi_VarArrayFetch(vfix,i);
    mv = Ddi_VarArrayFetch(vmov,i);
    fixpos = Ddi_VarCurrPos(dd,fv);
    movpos = Ddi_VarCurrPos(dd,mv);

    Pdtutil_Assert(abs(fixpos-movpos)==1,"grouping non adjacent variables");
    if (fixpos<movpos) {
      Cudd_MakeTreeNode(Ddi_MgrReadMgrCU(dd),Ddi_VarIndex(fv),2,MTR_DEFAULT);
    } else {
      Cudd_MakeTreeNode(Ddi_MgrReadMgrCU(dd),Ddi_VarIndex(mv),2,MTR_DEFAULT);
    }
  }

  return;
}

/**Function********************************************************************
  Synopsis    [Stores the variable ordering]
  Description [This function stores the variable ordering of a dd manager.
               Variable names and aux ids are used.]
  SideEffects [None]
  SeeAlso     []
******************************************************************************/

int
Ddi_MgrWriteOrd (
  Ddi_DdMgr *dd                              /* Decision Diagram Manager */,
  char *filename                             /* File Name */,
  FILE *fp                                   /* Pointer to the Store File */,
  Pdtutil_VariableOrderFormat_e fileFormat   /* File Format */
)
{
  char **vnames = Ddi_MgrReadVarnames (dd);
  int *vauxids = Ddi_MgrReadVarauxids (dd);
  int *iperm, nvars, i;

  nvars = Ddi_ReadNumVars (dd);
  iperm = Pdtutil_Alloc(int,nvars); 
  for (i=0; i<nvars; i++) {
    iperm[i] = Ddi_ReadInvPerm (dd,i);
  }

  Pdtutil_WriteOrd (vnames, vauxids, iperm, nvars, filename, fp,
    fileFormat);

  Pdtutil_Free(iperm);

  return (0);
}

/**Function********************************************************************
  Synopsis    [Reads the variable ordering]
  Description [This function reads the variable ordering of a dd manager.
               Existing variables with names in the ordering are shuffled
               to match the ordering.]
  SideEffects [None]
  SeeAlso     []
******************************************************************************/

int
Ddi_MgrReadOrdNamesAuxids (
  Ddi_DdMgr *dd               /* Manager */,
  char *filename              /* File Name */,
  FILE *fp                    /* Pointer to the Store File */
)
{
  int nvars, i, id, *sortedIds;
  Ddi_Var_t *var;
  char **vnames;
  int *vauxids;

  nvars = Pdtutil_ReadOrd (&vnames, &vauxids, filename, fp);

  if (nvars <= 0) 
    return (nvars);

  sortedIds = Pdtutil_Alloc(int,nvars); 
  Pdtutil_Assert(vnames!=NULL,"not yet supported read ord without varnames");
  if (vnames != NULL) {
    for (i=0; i<nvars; i++) {
      Pdtutil_Assert(vnames[i] != NULL, "NULL varname in ReadOrd");
      id = Ddi_VarName2Id(dd,vnames[i]);
      if (id < 0) { /* create a new var */
        var = Ddi_DdNewVar(dd);
        Ddi_VarSetName(dd,var,vnames[i]);
        if (vauxids != NULL)
          Ddi_VarSetAuxid(dd,var,vauxids[i]);
        id = Ddi_VarIndex(var);
      }
      sortedIds[i] = id;
    }
  }

  Ddi_MgrShuffle (dd, sortedIds, nvars);

  Pdtutil_Free(sortedIds);
  
  return nvars;
}


/**Function********************************************************************
  Synopsis     [Suspend autodyn if active.]
  SideEffects  [none]
  SeeAlso      [Ddi_MgrInit]
******************************************************************************/
void
Ddi_MgrAutodynSuspend(
  Ddi_DdMgr *dd  /* dd manager */
)
{
  Cudd_ReorderingType dynordMethod;
  int autodyn = Ddi_ReadReorderingStatus (dd, &dynordMethod);  

  /* actually suspend only if enabled */
  if (autodyn) {
    Ddi_MgrAutodynDisable (dd);
    dd->autodynMethod = dynordMethod; /* suspended autodyn method */
    dd->autodynSuspended = 1; 
  }

}


/**Function********************************************************************
  Synopsis     [Resume autodyn if suspended.]
  SideEffects  [none]
  SeeAlso      [Ddi_MgrInit]
******************************************************************************/
void
Ddi_MgrAutodynResume(
  Ddi_DdMgr *dd  /* dd manager */
  )
{
  if (dd->autodynSuspended) {
    Ddi_MgrAutodynEnable (dd, dd->autodynMethod);
  }

  return;
}



/**Function********************************************************************
  Synopsis     [Reads the Cudd Manager]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
DdManager *
Ddi_MgrReadMgrCU(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return (dd->mgrCU);
}

/**Function********************************************************************
  Synopsis     [Reads one constant]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
Ddi_Dd_t *
Ddi_MgrReadOne(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return dd->one;
}

/**Function********************************************************************
  Synopsis     [Reads zero constant]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
Ddi_Dd_t *
Ddi_MgrReadZero(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return dd->zero;
}

/**Function********************************************************************
  Synopsis     [Reads the variable names]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
char**
Ddi_MgrReadVarnames(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  int i;
  int nvars = Ddi_ReadNumVars(dd);

  if (dd->vararraySize<nvars) {
    dd->vararraySize *= 2;
    dd->varnames = Pdtutil_Realloc(char *, dd->varnames, dd->vararraySize);
    dd->varauxids = Pdtutil_Realloc(int, dd->varauxids, dd->vararraySize);
    for (i = dd->vararraySize/2; i<dd->vararraySize; i++) {
      dd->varnames[i] = NULL;
      dd->varauxids[i] = -1;
    }
  }

  return dd->varnames;
}

/**Function********************************************************************
  Synopsis     [Reads the variable auxiliary ids]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
int *
Ddi_MgrReadVarauxids(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  int i;
  int nvars = Ddi_ReadNumVars(dd);

  if (dd->vararraySize<nvars) {
    dd->vararraySize *= 2;
    dd->varnames = Pdtutil_Realloc(char *, dd->varnames, dd->vararraySize);
    dd->varauxids = Pdtutil_Realloc(int, dd->varauxids, dd->vararraySize);
    for (i = dd->vararraySize/2; i<dd->vararraySize; i++) {
      dd->varnames[i] = NULL;
      dd->varauxids[i] = -1;
    }
  }

  return dd->varauxids;
}

/**Function********************************************************************
  Synopsis     [Reads the counter of Dd Handles]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
int 
Ddi_MgrReadDdNum(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return dd->ddNum;
}

/**Function********************************************************************
  Synopsis     [Returns the threshold for the next dynamic reordering.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
unsigned int 
Ddi_MgrReadDynordThresh(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return Cudd_ReadNextReordering(Ddi_MgrReadMgrCU(dd));
}

/**Function********************************************************************
  Synopsis     [Returns the threshold for the next dynamic reordering.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

char *
Ddi_ReadDdiName (
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return (dd->ddiName);
}


/**Function********************************************************************
  Synopsis     [Returns the threshold for the next dynamic reordering.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

void
Ddi_SetDdiName (
  Ddi_DdMgr *dd            /* source dd manager */,
  char *ddiName
)
{
  dd->ddiName = Pdtutil_StrDup (ddiName);
}


/**Function********************************************************************
  Synopsis     [Read verbosity]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

Pdtutil_VerbLevel_e
Ddi_MgrReadVerbosity(
  Ddi_DdMgr *ddiMgr         /* Decision Diagram Interface Manager */
)
{
  return (ddiMgr->settings.verbosity);
}

/**Function********************************************************************
  Synopsis     [Set verbosity]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/

void
Ddi_MgrSetVerbosity(
  Ddi_DdMgr *ddiMgr              /* Decision Diagram Interface Manager */,
  Pdtutil_VerbLevel_e verbosity   /* Verbosity Level */
)
{
  ddiMgr->settings.verbosity = verbosity;

  return;
}



/**Function********************************************************************
  Synopsis     [Sets the CUDD manager]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
Ddi_MgrSetMgrCU(
  Ddi_DdMgr *dd            /* source dd manager */,
  DdManager *m             /* CUDD manager */
)
{
  dd->mgrCU=m;
  return;
}

/**Function********************************************************************
  Synopsis     [Sets the one constant]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
Ddi_MgrSetOne(
  Ddi_DdMgr *dd            /* source dd manager */,
  Ddi_Dd_t   *one          /* one constant */
)
{
  dd->one = one;
  return;
}

/**Function********************************************************************
  Synopsis     [Sets the zero constant]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
Ddi_MgrSetZero(
  Ddi_DdMgr *dd            /* source dd manager */,
  Ddi_Dd_t   *zero          /* zero constant */
)
{
  dd->zero = zero;
  return;
}

/**Function********************************************************************
  Synopsis     [Sets the names of variables]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
Ddi_MgrSetVarnames(
  Ddi_DdMgr *dd            /* source dd manager */,
  char      **vn           /* names of variables */
)
{

printf("set varnames: funzione da eliminare!\n");
  dd->varnames=vn;
  return;
}

/**Function********************************************************************
  Synopsis     [Sets the auxiliary variable ids]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
Ddi_MgrSetVarauxids(
  Ddi_DdMgr *dd            /* source dd manager */,
  int       *va            /* auxiliary variable ids */
)
{
  dd->varauxids=va;
  return;
}

/**Function********************************************************************
  Synopsis     [Returns the threshold for the next dynamic reordering.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
void
Ddi_MgrSetDynordThresh(
  Ddi_DdMgr *dd            /* source dd manager */,
  unsigned int th           /* threshold */
)
{
  Cudd_SetNextReordering(Ddi_MgrReadMgrCU(dd), th);
  return;
}

/*
*********************** Computed Table (Cache) ********************************
*/

/**Function********************************************************************
  Synopsis     [Reads the number of slots in the cache.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
unsigned int
Ddi_ReadCacheSlots(
  Ddi_DdMgr *dd            /* source dd manager */
)
{
  return Cudd_ReadCacheSlots(Ddi_MgrReadMgrCU(dd));
}

/**Function********************************************************************
  Synopsis     [Returns the number of cache look-ups.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
double
Ddi_ReadCacheLookUps(
  Ddi_DdMgr *dd            /* dd manager */
)
{
  /* DM : questo campo non esiste pi, ma viene calcolato come
  * somma di altri campi 
  */
  return Cudd_ReadCacheLookUps(Ddi_MgrReadMgrCU(dd));
}

/**Function********************************************************************
  Synopsis     [Returns the number of cache hits.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
double
Ddi_ReadCacheHits(
  Ddi_DdMgr *dd            /* dd manager */
)
{
  /* DM : fornisce cacheHits+totCacheHits anzich
  * solo cacheHits previsto dalla define
  */
  return Cudd_ReadCacheHits(Ddi_MgrReadMgrCU(dd));
}

/**Function********************************************************************
  Synopsis     [Reads the hit ratio that causes resizing of the computed
  table.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
unsigned int
Ddi_ReadMinHit(
  Ddi_DdMgr *dd            /* dd manager */
)
{
  /* DM : fornisce un'espressione razionale di minHit anzich
  * solo minHit previsto dalla define
  */
  return Cudd_ReadMinHit(Ddi_MgrReadMgrCU(dd));
}

/**Function********************************************************************
  Synopsis     [Reads the maxCacheHard parameter of the manager.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
unsigned int
Ddi_ReadMaxCacheHard(
  Ddi_DdMgr *dd            /* dd manager */
)
{
  return Cudd_ReadMaxCacheHard(Ddi_MgrReadMgrCU(dd));
}

/**Function********************************************************************
  Synopsis     [Returns the soft limit for the cache size.]
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
unsigned int
Ddi_ReadMaxCache(
  Ddi_DdMgr* dd            /* dd manager */
)
{
  /* DM : questo campo non esiste pi, ma viene calcolato come
  * somma di altri campi 
  */
  return Cudd_ReadMaxCache( Ddi_MgrReadMgrCU( dd ) );
}

