/**CFile***********************************************************************

  FileName    [ddiNew.c]

  PackageName [ddi]

  Synopsis    [Utility Functions]

  Description []

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
    Torino, Italy.
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.
    ]
  
  Revision    []

******************************************************************************/

/*---------------------------------------------------------------------------*/
/* Nested includes                                                           */
/*---------------------------------------------------------------------------*/

#include "ddiInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/


/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis           []

  Description        []

  SideEffects        []

  SeeAlso            []

******************************************************************************/

int
Ddi_BddOperation (
  Ddi_DdMgr *defaultDdMgr         /* Default DD Manager */,
  Ddi_Dd_t **bddP                 /* BDD Pointer to manipulate */,
  char *string                    /* String */,
  Pdtutil_MgrOp_t operationFlag    /* Operation Flag */,
  void **voidPointer              /* Generic Pointer */,
  Pdtutil_MgrRet_t *returnFlag     /* Type of the Pointer Returned */
  )
{
  char *stringFirst, *stringSecond;
  Ddi_DdMgr *tmpDdMgr;

  /*----------------------- Take Main and Secondary Name --------------------*/

  Pdtutil_ReadSubstring (string, &stringFirst, &stringSecond);

  /*------------------------ Operation on the BDD Manager -------------------*/

  if (stringFirst == NULL) {
    switch (operationFlag) {
      case Pdtutil_MgrOpRead_c:
      case Pdtutil_MgrOpBddRead_c:
        *voidPointer = (void *) *bddP;
        *returnFlag = Pdtutil_MgrRetBdd_c;
        break;

      case Pdtutil_MgrOpDelete_c:
      case Pdtutil_MgrOpBddDelete_c:
        if (*bddP != NULL) {
          Ddi_DdFree (*bddP);
        }
        *bddP = NULL;
        break;

      case Pdtutil_MgrOpSet_c:
      case Pdtutil_MgrOpBddSet_c:
        if (*bddP != NULL) {
          Ddi_DdFree (*bddP);
        }
        *bddP = Ddi_DdCopy (defaultDdMgr, (Ddi_Dd_t *) *voidPointer);
        break;

     case Pdtutil_MgrOpStats_c: 
        Ddi_DdPrintStats ((Ddi_Dd_t *) *bddP);
        break;

      case Pdtutil_MgrOpMgrRead_c:
        tmpDdMgr = Ddi_DdReadMgr (*bddP);
        *voidPointer = (void *) tmpDdMgr;
        *returnFlag = Pdtutil_MgrRetDdMgr_c;
        break;

      case Pdtutil_MgrOpMgrSet_c:
        tmpDdMgr = (Ddi_DdMgr *) *voidPointer;
        Ddi_DdSetMgr (*bddP, tmpDdMgr);
        break;

      default:
        Pdtutil_Warning (1, "Operation Non Allowed on BDD Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*------------------------ Operation on the BDD Manager -------------------*/

  if (strcmp (stringFirst, "ddm") == 0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpMgrRead_c:
        tmpDdMgr = Ddi_DdReadMgr (*bddP);
        *voidPointer = (void *) tmpDdMgr;
        *returnFlag = Pdtutil_MgrRetDdMgr_c;
        break;

      case Pdtutil_MgrOpMgrSet_c:
        tmpDdMgr = (Ddi_DdMgr *) *voidPointer;
        Ddi_DdSetMgr (*bddP, tmpDdMgr);
        break;

      default:
        Pdtutil_Warning (1, "Operation Non Allowed on BDD Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*------------------------------- No Match --------------------------------*/

  Pdtutil_Warning (1, "Operation on BDD not allowed");
  return (1);
}

/**Function********************************************************************

  Synopsis           []

  Description        []

  SideEffects        []

  SeeAlso            []

******************************************************************************/

int
Ddi_BddArrayOperation (
  Ddi_DdMgr *defaultDdMgr         /* Default BDD Manager */,
  Ddi_DdArray_t **bddArrayP       /* BDD Array Pointer to manipulate */,
  char *string                    /* String */,
  Pdtutil_MgrOp_t operationFlag    /* Operation Flag */,
  void **voidPointer              /* Generic Pointer */,
  Pdtutil_MgrRet_t *returnFlag     /* Type of the Pointer Returned */
  )
{
  int i;
  char *stringFirst, *stringSecond;
  Ddi_Dd_t *bdd;

  /*----------------------- Take Main and Secondary Name --------------------*/

  Pdtutil_ReadSubstring (string, &stringFirst, &stringSecond);

  /*------------------------ Operation on the BDD Manager -------------------*/

  if (stringFirst == NULL) {
    switch (operationFlag) {
      case Pdtutil_MgrOpDelete_c:
      case Pdtutil_MgrOpArrayDelete_c:
        if (*bddArrayP != NULL) {
          Ddi_DdArrayFree (*bddArrayP);
        }
        *bddArrayP = NULL;
        break;

      case Pdtutil_MgrOpRead_c:
      case Pdtutil_MgrOpArrayRead_c:
        *voidPointer = (void *) *bddArrayP;
        *returnFlag = Pdtutil_MgrRetBddArray_c;
        break;

      case Pdtutil_MgrOpSet_c:
      case Pdtutil_MgrOpArraySet_c:
        if (*bddArrayP != NULL) {
          Ddi_DdArrayFree (*bddArrayP);
        }
        *bddArrayP = Ddi_DdArrayCopy (defaultDdMgr,
	  (Ddi_DdArray_t *) *voidPointer);
        break;

      case Pdtutil_MgrOpStats_c:
        Ddi_DdArrayPrintStats ((Ddi_DdArray_t *) *bddArrayP);
        break;

      default:
        Pdtutil_Warning (1, "Operation Non Allowed on BDD Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*-------------------------- Operation on one Element ---------------------*/

  if (stringSecond == NULL) {
    i = atoi (stringFirst);
    if ( (i>=0) && (i<Ddi_DdArray_n (*bddArrayP)) ) {
      bdd = Ddi_DdArrayFetch (*bddArrayP, i);
      Ddi_BddOperation (defaultDdMgr, &bdd, stringSecond,
        operationFlag, voidPointer, returnFlag); 
      return (0);
    } else {
      Pdtutil_Warning (1,
        "Operation on BDD not allowed or Array index out of bounds");
      *returnFlag = Pdtutil_MgrRetNone_c;
      return (1);
    }
  }

  /*------------------------------- No Match --------------------------------*/

  Pdtutil_Warning (1, "Operation on BDD not allowed");
  return (1);
}



/**Function********************************************************************
 Synopsis    [Reads a cube from stdin]
 Description [The user can make a cube typing the index of variables]
 SideEffects [none]
 SeeAlso     []
******************************************************************************/

Ddi_Dd_t *
Ddi_ReadCube(
  Ddi_DdMgr *dd,
  FILE *fp,
  int idOrName
)
{
  Ddi_Dd_t *cube,*V,*V2;
  int n, n2, compl, op2;
  char buf[100], *name, name1[100], name2[100];

  op2 = 0;
  cube = Ddi_MgrReadOne(dd);
  while (fgets(buf,100,fp)!=NULL) {
    if (buf[0] == '#') continue; /* comment */
    if (buf[0]=='!') {
      compl = 1;
      name = &buf[1];
    }
    else {
      compl = 0;
      name = buf;
    }
    /* load from file */
    if (name[0]=='$') {
      name++;
      sscanf(name,"%s",name1); /* removes \n */
      printf("loading constrain from %s\n",name1);
      V = Ddi_DdLoad(dd,DDDMP_VAR_MATCHNAMES,
          DDDMP_MODE_DEFAULT,name1,NULL);  
      if (compl)
        V = Ddi_DdEvalFree(Ddi_DdNot(V),V);
    }
    else {
      if ((name[0]=='=')||(name[0]=='&')||(name[0]=='|')) {
        op2 = name[0];
        name++;
      }
      if (idOrName) {
        if (op2)
          sscanf(name,"%d %d",&n, &n2);
        else
          sscanf(name,"%d",&n);
      }
      else {
        while(name[0]==' ') name++;
        sscanf(name,"%s",name1);
        n = Ddi_VarName2Id(dd,name1);
        if (op2) {
          sscanf(name,"%*s %s",name2);
          n2 = Ddi_VarName2Id(dd,name2);
        }
        /* occorre restituire NULL ?*/
        if (n<0) 
          fprintf(stderr, "Error : Variable name not found for cube: %s\n",name);
      }
      if (n <= 0)
        break;
      V = Ddi_DdLiteral(dd, Ddi_IthVar (dd,n));
      if (op2) {
        V2 = Ddi_DdLiteral(dd, Ddi_IthVar (dd,n2));
        switch (op2) {
          case '&':          
            V = Ddi_DdEvalFree(Ddi_DdAnd(V,V2),V);
            break;
          case '|':          
            V = Ddi_DdEvalFree(Ddi_DdOr(V,V2),V);
            break;
          case '=':          
            V = Ddi_DdEvalFree(Ddi_DdXnor(V,V2),V);
            break;
          default:
            printf ("unknown operator in cube file %c\n",op2);
        }
        Ddi_DdFree(V2);
      }
    }  
    if (compl)
      V = Ddi_DdEvalFree(Ddi_DdNot(V),V);
 
    cube = Ddi_DdEvalFree(Ddi_DdAnd(cube,V),cube);
    Ddi_DdFree(V);
  }

  return cube;
}




/**Function********************************************************************
 Synopsis    [Reads a variable set from file]
 Description [The user can make a cube typing the index of variables]
 SideEffects [none]
 SeeAlso     []
******************************************************************************/

Ddi_VarSet_t *
Ddi_ReadVarSet(
  Ddi_DdMgr *dd,
  FILE *fp,
  int idOrName
)
{
  Ddi_VarSet_t *vars;
  int n, op2;
  char buf[PDUTIL_MAX_STR_LEN];

  op2 = 0;
  vars = Ddi_VarSetVoid (dd);
  while (fscanf(fp,"%s",buf)>0) {
    if (idOrName) 
      sscanf(buf,"%d",&n);
    else
      n = Ddi_VarName2Id(dd,buf);
    vars = Ddi_VarSetEvalFree(
             Ddi_VarSetAdd (vars, Ddi_IthVar (dd,n)),
           vars);
  }
  return vars;
}


/**Function********************************************************************
 Synopsis    [The function does a substitution of variables of a BDD]
 Description [The function returns the pointer to a new function with the
              variables swapped (x replace y), or NULL otherwise.]
 SideEffects [none]
 SeeAlso     []
******************************************************************************/
Ddi_Dd_t *
Ddi_VarSubst(
  Ddi_Dd_t *f         /* BDD */,
  Ddi_VarArray_t *x   /* first array of variables */,
  Ddi_VarArray_t *y   /* second array of variables */
)
{
#if 1
  Pdtutil_Warning (1, "Not implemented Pdtutil_VarSubst.");
  return (NULL);
#else
  Ddi_Dd_t *f0,*f1,*xi,*yi, *newf;
  Ddi_VarSet_t *supp,*supp1;
  int i;
 
  newf = Ddi_DdDup(f);

  supp = Ddi_DdSupport(newf);

  for (i=0; i<Ddi_VarArray_n(x); i++) {
    xi = Ddi_VarArrayFetch(x,i);
    supp1 = Ddi_DdConstrain(supp,xi);
    if (Ddi_VarSetequal(supp1,supp)) {
      Ddi_DdFree(supp1);
      continue;
    }
    Ddi_DdFree(supp1);

    yi = Ddi_VarArrayFetch(y,i);
    f0 = Ddi_DdConstrain(newf,Ddi_Not(xi));
    f1 = Ddi_DdConstrain(newf,xi);
    Ddi_DdFree(newf);
    newf = Ddi_DdIte(yi,f1,f0);
    Ddi_DdFree(f0);
    Ddi_DdFree(f1);
  }

  Ddi_DdFree(supp);

  return newf;
#endif
}


/**Function********************************************************************
  Synopsis    [Prints the indices of a VarArray]
  SideEffects [None]
  SeeAlso     [Ddi_PrintDdArray]
******************************************************************************/

void
Ddi_PrintVarArray (
  Ddi_VarArray_t *array )
{
  int i;

  for (i=0;i<Ddi_VarArray_n(array);i++) 
    printf("%d ",Ddi_VarArrayIndex(array,i));
  printf("\n");
  fflush(stdout);
}


/**Function********************************************************************
  Synopsis    [Prints the size of each function in a DdArray]
  SideEffects [None]
  SeeAlso     [Ddi_PrintVarArray]
******************************************************************************/
void
Ddi_PrintDdArray (
  Ddi_DdArray_t *array )
{
  int i;
  Ddi_Dd_t *F;

  for (i=0;i<Ddi_DdArray_n(array);i++) {
    F = Ddi_DdArrayFetch(array,i);
    printf("%d ",Ddi_DdSize(F));
  }
  printf("\n");
  fflush(stdout);
}

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Definition of static functions                                            */
/*---------------------------------------------------------------------------*/


