/**CFile***********************************************************************

  FileName    [ddiVar.c]

  PackageName [ddi]

  Synopsis    [Functions to manipulate BDD variables]

  Description []

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
  Torino, Italy. 
  The  Politecnico di Torino makes no warranty about the suitability of 
  this software for any purpose.  
  It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#include "ddiInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/
  
/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/
  
/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/


/**Function********************************************************************
  Synopsis    [Returns the variable index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
int
Ddi_VarIndex (
  Ddi_Var_t *var )
{
  return ((int)Cudd_NodeReadIndex(Ddi_VarToCU(var)));
}


/**Function********************************************************************
  Synopsis    [Returns the variable of a given index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
Ddi_Var_t *
Ddi_IthVar (
  Ddi_DdMgr   *dd,
  int index 
)
{
  DdNode *varCU;

  varCU = Cudd_bddIthVar(dd->mgrCU,index);
  if (varCU->ref == 0)
    Cudd_Ref(varCU);

  DdiMgrCheckVarArraySize(dd);

  return (Ddi_VarFromCU(varCU));
}


/**Function********************************************************************
  Synopsis    [Returns a new variable]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
Ddi_Var_t *
Ddi_DdNewVar(
  Ddi_DdMgr   *dd
)
{
  DdNode *varCU;

  varCU = Cudd_bddNewVar(Ddi_MgrReadMgrCU(dd));
  if (varCU->ref == 0)
    Cudd_Ref(varCU);

  DdiMgrCheckVarArraySize(dd);

  return (Ddi_VarFromCU(varCU));
}


/**Function********************************************************************
  Synopsis    [Returns a new variable at a given level in the order]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
Ddi_Var_t *
Ddi_DdNewVarAtLevel(
  Ddi_DdMgr   *dd,
  int          lev
)
{
  DdNode *varCU;

  varCU = Cudd_bddNewVarAtLevel(Ddi_MgrReadMgrCU(dd),lev);
  if (varCU->ref == 0)
    Cudd_Ref(varCU);

  DdiMgrCheckVarArraySize(dd);

  return (Ddi_VarFromCU(varCU));
}


/**Function********************************************************************
  Synopsis    [Returns the current position of the variable in the order]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
int
Ddi_VarCurrPos (
  Ddi_DdMgr   *dd,
  Ddi_Var_t *var
)
{
  return Cudd_ReadPerm(dd->mgrCU,Ddi_VarIndex(var));
}

/**Function********************************************************************
  Synopsis    [Returns the variable name given the index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
char *
Ddi_VarId2Name (
  Ddi_DdMgr   *dd,
  int index
)
{
  char **varnames=Ddi_MgrReadVarnames(dd);
  if (varnames==NULL)
    return NULL;
  return (varnames[index]);
}

/**Function********************************************************************
  Synopsis    [Returns the variable name given the index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
char *
Ddi_VarName (
  Ddi_DdMgr   *dd,
  Ddi_Var_t *var 
)
{
  char **varnames=Ddi_MgrReadVarnames(dd);
  if (varnames==NULL)
    return NULL;
  return (varnames[Ddi_VarIndex(var)]);
}

/**Function********************************************************************
  Synopsis    [Returns the variable name given the index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
void
Ddi_VarSetName (
  Ddi_DdMgr   *dd,
  Ddi_Var_t *var,
  char *name
)
{
  int index;
  char **varnames;

  DdiMgrCheckVarArraySize(dd);

  varnames=Ddi_MgrReadVarnames(dd);
  Pdtutil_Assert(varnames!=NULL, "null varnames array in dd manager");

  index = Ddi_VarIndex(var);
  Pdtutil_Assert(varnames[index]==NULL,"setting name to a named variable");
  varnames[index] = Pdtutil_StrDup(name);
}

/**Function********************************************************************
  Synopsis    [Returns the variable auxid given the index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
int
Ddi_VarId2Auxid (
  Ddi_DdMgr   *dd,
  int index
)
{
  int *varauxids=Ddi_MgrReadVarauxids(dd);
  if (varauxids==NULL)
    return -1;
  return (varauxids[index]);
}

/**Function********************************************************************
  Synopsis    [Returns the variable auxid]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
int
Ddi_VarAuxid (
  Ddi_DdMgr   *dd,
  Ddi_Var_t *var
)
{
  int index = Ddi_VarIndex(var);
  int *varauxids=Ddi_MgrReadVarauxids(dd);
  if (varauxids==NULL)
    return -1;
  return (varauxids[index]);
}

/**Function********************************************************************
  Synopsis    [Returns the variable name given the index]
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
void
Ddi_VarSetAuxid (
  Ddi_DdMgr   *dd,
  Ddi_Var_t *var,
  int auxid
)
{
  int index;
  int *varauxids=Ddi_MgrReadVarauxids(dd);
  Pdtutil_Assert(varauxids!=NULL, "null varauxids array in dd manager");

  index = Ddi_VarIndex(var);
  Pdtutil_Assert(varauxids[index]==-1,"setting auxid to a variable twice");
  varauxids[index] = auxid;
}

/**Function********************************************************************
  Synopsis    [Returns the variable index given the name]
  Description [Still a linear search !] 
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
int
Ddi_VarName2Id (
  Ddi_DdMgr *dd,
  char *name
)
{
  int i;
  char **varnames=Ddi_MgrReadVarnames(dd);

  if (varnames==NULL) {
    return (-1);
  }

  for (i=0;i<Ddi_ReadSize(dd);i++) {
    if ((varnames[i]!=NULL) && (strcmp(varnames[i],name)==0)) {
      return (i);
    }
  }

  return (-1);
}

/**Function********************************************************************
  Synopsis    [Returns the variable index given the auxid]
  Description [Still a linear search !] 
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
int
Ddi_VarAuxid2Id (
  Ddi_DdMgr   *dd,
  int auxid
)
{
  int i;
  int *varauxids=Ddi_MgrReadVarauxids(dd);
  if (varauxids==NULL)
    return -1;

  for (i=0;i<Ddi_ReadSize(dd);i++) {
    if (varauxids[i]==auxid)
      return i;
  }
  return -1;
}

/**Function********************************************************************
  Synopsis    [Converts a variable into a Cudd node]
  Description [] 
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
DdNode *
Ddi_VarToCU(
  Ddi_Var_t * v
)
{
  return (DdNode *)v; 
}

/**Function********************************************************************
  Synopsis    [Converts a Cudd node into a variable]
  Description [] 
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
Ddi_Var_t *
Ddi_VarFromCU(
  DdNode *  v
)
{
  return (Ddi_Var_t *)v; 
}

/**Function********************************************************************
  Synopsis     [Copies a variable to a destination dd manager]
  Description  [Finds the variable corresponding to v in the destination
               manager. Variable correspondence is for now limited to
               index matching.] 
  SideEffects  [none]
  SeeAlso      []
******************************************************************************/
Ddi_Var_t *
Ddi_VarCopy(
  Ddi_DdMgr *dd2 /* destination manager */,
  Ddi_Var_t *v    /* variable to be copied */ 
)
{
  Ddi_Var_t *v2;
  int id;

  id = Ddi_VarIndex(v);
  v2 = Ddi_IthVar(dd2,id);

  return (v2);
}

/**Function********************************************************************
  Synopsis    [Create a variable group]
  Description [A group of variables is created for group sifting] 
  SideEffects [none]
  SeeAlso     []
******************************************************************************/
void
Ddi_VarMakeGroup(
  Ddi_DdMgr   *dd,
  Ddi_Var_t   *v,
  int grpSize
)
{
  Cudd_MakeTreeNode(dd->mgrCU,Ddi_VarIndex(v),grpSize,MTR_DEFAULT);
}

