/**CFile***********************************************************************

  FileName    [fsmMgr.c]

  PackageName [fsm]

  Synopsis    [Utility functions to create, free and duplicate a FSM]

  Description []

  SeeAlso     []  

  Author      [Gianpiero Cabodi and Stefano Quer]

  Copyright   [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#include "fsmInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/


/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis     [Initializes the FSM structure.]

  Description  []

  SideEffects  [none]

******************************************************************************/

Fsm_Mgr_t *
Fsm_MgrInit (
  char *fsmName   /* Name of the FSM structure */
  )
{
  Fsm_Mgr_t *fsmMgr;

  /*------------------------- FSM Structure Creation -----------------------*/

  fsmMgr = Pdtutil_Alloc (Fsm_Mgr_t, 1);
  if (fsmMgr == NULL) {
    fprintf (stderr, "fsmUtil.c (Fsm_MgrInit) Error: Out of memory.\n");
    return (NULL);
  }

  /*------------------- Set the Name of the FSM Structure ------------------*/

  if (fsmName != NULL) {
     Fsm_MgrSetFsmName (fsmMgr, fsmName);
  } else {
     Fsm_MgrSetFsmName (fsmMgr, "fsm");
  }

  /*------------------------------- Set Fields -----------------------------*/

  Fsm_MgrSetFileName (fsmMgr, (char *) NULL);
  Fsm_MgrSetDdManager (fsmMgr, NULL);

  Fsm_MgrSetNI (fsmMgr, 0);
  Fsm_MgrSetNO (fsmMgr, 0);
  Fsm_MgrSetNL (fsmMgr, 0);

  Fsm_MgrSetY (fsmMgr, NULL);
  Fsm_MgrSetSX (fsmMgr, NULL);
  Fsm_MgrSetZ (fsmMgr, NULL);

  Fsm_MgrSetVarnames (fsmMgr, NULL);
  Fsm_MgrSetVarauxids (fsmMgr, NULL);
  Fsm_MgrSetInvauxids (fsmMgr, NULL);

  Fsm_MgrSetIndexI (fsmMgr, NULL);
  Fsm_MgrSetIndexO (fsmMgr, NULL);
  Fsm_MgrSetIndexPS (fsmMgr, NULL);
  Fsm_MgrSetIndexNS (fsmMgr, NULL);

  Fsm_MgrSetNameI (fsmMgr, NULL);
  Fsm_MgrSetNameO (fsmMgr, NULL);
  Fsm_MgrSetNamePS (fsmMgr, NULL);
  Fsm_MgrSetNameNS (fsmMgr, NULL);

  Fsm_MgrSetBddFormat (fsmMgr, DDDMP_MODE_TEXT);

  Fsm_MgrSetDeltaName (fsmMgr, NULL);
  Fsm_MgrSetDeltaBDD (fsmMgr, NULL);
  Fsm_MgrSetLambdaName (fsmMgr, NULL);
  Fsm_MgrSetLambdaBDD (fsmMgr, NULL);

  Fsm_MgrSetInitName (fsmMgr, NULL);
  Fsm_MgrSetInitString (fsmMgr, NULL);
  Fsm_MgrSetInitBDD (fsmMgr, NULL);
  Fsm_MgrSetOrdNodeName (fsmMgr, NULL);
  Fsm_MgrSetOrdFileName (fsmMgr, NULL);
  Fsm_MgrSetTrName (fsmMgr, NULL);
  Fsm_MgrSetTrString (fsmMgr, NULL);
  Fsm_MgrSetTrBDD (fsmMgr, NULL);
  Fsm_MgrSetFromName (fsmMgr, NULL);
  Fsm_MgrSetFromString (fsmMgr, NULL);
  Fsm_MgrSetFromBDD (fsmMgr, NULL);
  Fsm_MgrSetReachedName (fsmMgr, NULL);
  Fsm_MgrSetReachedString (fsmMgr, NULL);
  Fsm_MgrSetReachedBDD (fsmMgr, NULL);

  /*
   *  Settings
   */

  fsmMgr->fsmTime = 0;
  Fsm_MgrSetVerbosity (fsmMgr, Pdtutil_VerbLevelNone_c);

  return (fsmMgr);
}

/**Function********************************************************************

  Synopsis     [Frees unused FSM structure members ]

  Description  []

  SideEffects  [none]

  SeeAlso      [Ddi_DdArrayFree();Ddi_VarArrayFree()]

******************************************************************************/

void 
Fsm_MgrFree (
  Fsm_Mgr_t *fsmMgr   /* FSM Manager */
)
{
  /*------------------------ Deletes the FSM Manager -------------------*/

  /* Free fsmName */
  Pdtutil_Free (fsmMgr->fsmName);

  /* Free fileName */
  Pdtutil_Free (fsmMgr->fileName);

  /* var */
  Ddi_VarArrayFree (Fsm_MgrReadVarI (fsmMgr));
  Ddi_VarArrayFree (Fsm_MgrReadVarNS (fsmMgr));
  Ddi_VarArrayFree (Fsm_MgrReadVarPS (fsmMgr));

  /* y, sx and z */
  Ddi_VarArrayFree(Fsm_MgrReadY(fsmMgr));  
  Ddi_VarArrayFree(Fsm_MgrReadZ(fsmMgr));  
  Ddi_VarArrayFree(Fsm_MgrReadSX(fsmMgr));
  
  /* Free varnames */
  Pdtutil_StrArrayFree(Fsm_MgrReadVarnames(fsmMgr),
    Fsm_MgrReadNI (fsmMgr)+2*Fsm_MgrReadNL (fsmMgr));
 
  /* Free varauxids */
  Pdtutil_Free (fsmMgr->varauxids);

  /* Free invauxids */
  Pdtutil_Free (fsmMgr->invauxids);

  /* Free index */
  Pdtutil_Free (fsmMgr->index.i);
  Pdtutil_Free (fsmMgr->index.o);
  Pdtutil_Free (fsmMgr->index.ps);
  Pdtutil_Free (fsmMgr->index.ns);

  /* Free ord */
  Pdtutil_Free (fsmMgr->ord.name);
 
  Pdtutil_StrArrayFree (fsmMgr->ord.nodename,
    Fsm_MgrReadNI (fsmMgr)+2*Fsm_MgrReadNL (fsmMgr));
  Pdtutil_Free (fsmMgr->ord.nodename);

  Pdtutil_Free (fsmMgr->ord.nodeid);

  /* name */
  Pdtutil_Free (fsmMgr->name.i);
  Pdtutil_Free (fsmMgr->name.o);
  Pdtutil_Free (fsmMgr->name.ns);
  Pdtutil_Free (fsmMgr->name.ps);

  /* delta */
  Ddi_DdArrayFree (Fsm_MgrReadDeltaBDD(fsmMgr)); 

  /* lambda */
  Ddi_DdArrayFree (Fsm_MgrReadLambdaBDD (fsmMgr)); 
  
  /* init */
  Ddi_DdFree(Fsm_MgrReadInitBDD (fsmMgr));

  /* tr */
  Ddi_DdFree(Fsm_MgrReadTrBDD (fsmMgr));

  /* from */
  Ddi_DdFree(Fsm_MgrReadFromBDD (fsmMgr));
 
  /* reached */
  Ddi_DdFree(Fsm_MgrReadReachedBDD (fsmMgr));
  
  /* structure */
  Pdtutil_Free(fsmMgr);

  return;
}

/**Function********************************************************************

  Synopsis    [Duplicates FSM structure.]

  Description []

  SideEffects [none]

  SeeAlso     [Fsm_MgrInit ()]

******************************************************************************/

Fsm_Mgr_t *
Fsm_MgrDup (
  Fsm_Mgr_t *fsmMgr   /* FSM Manager */
)
{
  Fsm_Mgr_t *fsmMgrNew;

  fprintf (stderr, "fsmUtil.c: Warning Fsm_MgrDup till to test!\n");

  /*------------------------- FSM structure creation -----------------------*/

  fsmMgrNew = Fsm_MgrInit (NULL);

  /*------------------------------- Copy Fields ----------------------------*/

  Fsm_MgrSetDdManager (fsmMgrNew, Fsm_MgrReadDdManager (fsmMgr));
  
  if (Fsm_MgrReadFileName (fsmMgr) != NULL) {
    Fsm_MgrSetFileName (fsmMgrNew, Fsm_MgrReadFileName (fsmMgr));
  }

  if (Fsm_MgrReadY (fsmMgr) != NULL) {
    Fsm_MgrSetY (fsmMgrNew, Ddi_VarArrayDup (Fsm_MgrReadY (fsmMgr)));
  }
  if (Fsm_MgrReadZ (fsmMgr) != NULL) {
    Fsm_MgrSetZ (fsmMgrNew, Ddi_VarArrayDup (Fsm_MgrReadZ (fsmMgr)));
  }  
  if (Fsm_MgrReadSX (fsmMgr) != NULL) {
    Fsm_MgrSetSX (fsmMgrNew, Ddi_VarArrayDup (Fsm_MgrReadSX (fsmMgr)));
  }

  Fsm_MgrSetReachedBDD (fsmMgrNew, Fsm_MgrReadReachedBDD (fsmMgr));

  Fsm_MgrSetBddFormat (fsmMgrNew, Fsm_MgrReadBddFormat (fsmMgr));
  
  if (Fsm_MgrReadDeltaBDD (fsmMgr) != NULL) {
    Fsm_MgrSetDeltaBDD (fsmMgrNew, 
     Ddi_DdArrayDup(Fsm_MgrReadDeltaBDD (fsmMgr)));
  }
  if (Fsm_MgrReadLambdaBDD (fsmMgr) != NULL) {
    Fsm_MgrSetLambdaBDD (fsmMgrNew,
      Ddi_DdArrayDup(Fsm_MgrReadLambdaBDD (fsmMgr)));
  }
  if (Fsm_MgrReadTrBDD (fsmMgr) != NULL) {  
    Fsm_MgrSetTrBDD (fsmMgrNew, Ddi_DdDup(Fsm_MgrReadTrBDD (fsmMgr)));
  }
  if (Fsm_MgrReadInitBDD (fsmMgr) != NULL) {  
    Fsm_MgrSetInitBDD (fsmMgrNew, Ddi_DdDup (Fsm_MgrReadInitBDD (fsmMgr)));
  }
  
  if (Fsm_MgrReadDeltaName (fsmMgr) != NULL) {
    Fsm_MgrSetDeltaName (fsmMgrNew, Fsm_MgrReadDeltaName (fsmMgr));
  }
  if (Fsm_MgrReadLambdaName (fsmMgr) != NULL) {
    Fsm_MgrSetLambdaName (fsmMgrNew, Fsm_MgrReadLambdaName (fsmMgr));
  }
  fprintf (stderr, "fsmUtil.c: Warning Some Dup not supported yet!\n");
  /*
  if (Fsm_MgrReadTrName (fsmMgr) != NULL) {
    Fsm_MgrSetTrName (fsmMgrNew, Fsm_MgrReadTrName (fsmMgr));
  }
  */
  if (Fsm_MgrReadInitString (fsmMgr) != NULL) {
    Fsm_MgrSetInitString (fsmMgrNew, Fsm_MgrReadInitString (fsmMgr));
  }
  if (Fsm_MgrReadInitName (fsmMgr) != NULL) {
    Fsm_MgrSetInitName (fsmMgrNew, Fsm_MgrReadInitName (fsmMgr));
  }

  Fsm_MgrSetNI (fsmMgrNew, Fsm_MgrReadNI (fsmMgr));
  Fsm_MgrSetNO (fsmMgrNew, Fsm_MgrReadNO (fsmMgr));
  Fsm_MgrSetNL (fsmMgrNew, Fsm_MgrReadNL (fsmMgr));

  fprintf (stderr, "fsmUtil.c: Warning Some Dup not supported yet!\n");
  /*
  if (Fsm_MgrReadNameI (fsmMgr) != NULL) {
    Fsm_MgrSetNameI (fsmMgrNew, Fsm_MgrReadNameI (fsmMgr));
  }
  if (Fsm_MgrReadNameO (fsmMgr) != NULL) {
    Fsm_MgrSetNameO (fsmMgrNew, Fsm_MgrReadNameO (fsmMgr));
  }
  if (Fsm_MgrReadNamePS (fsmMgr) != NULL) {
    Fsm_MgrSetNamePS (fsmMgrNew, Fsm_MgrReadNamePS (fsmMgr));
  }
  if (Fsm_MgrReadNameNS (fsmMgr) != NULL) {
    Fsm_MgrSetNameNS (fsmMgrNew, Fsm_MgrReadNameNS (fsmMgr));
  }
  if (Fsm_MgrReadIndexI (fsmMgr) != NULL) {
    Fsm_MgrSetIndexI (fsmMgrNew, Fsm_MgrReadIndexI (fsmMgr));
  }
  if (Fsm_MgrReadIndexO (fsmMgr) != NULL) {
    Fsm_MgrSetIndexO (fsmMgrNew, Fsm_MgrReadIndexO (fsmMgr));
  }
  if (Fsm_MgrReadIndexPS (fsmMgr) != NULL) {
    Fsm_MgrSetIndexPS (fsmMgrNew, Fsm_MgrReadIndexPS (fsmMgr));
  }
  if (Fsm_MgrReadIndexNS (fsmMgr) != NULL) {
    Fsm_MgrSetIndexNS (fsmMgrNew, Fsm_MgrReadIndexNS (fsmMgr));
  }
  */

  return (fsmMgrNew);
}


/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     [CmdMgrOperation, CmdRegOperation, Trav_MgrOperation,
    Tr_MgrOperation]

******************************************************************************/

int
Fsm_MgrOperation (
  Fsm_Mgr_t *fsmMgr               /* FSM Manager */,
  char *string                    /* String */,
  Pdtutil_MgrOp_t operationFlag   /* Operation Flag */,
  void **voidPointer              /* Generic Pointer */,
  Pdtutil_MgrRet_t *returnFlagP   /* Type of the Pointer Returned */
  )
{ 
  char *stringFirst, *stringSecond;
  Ddi_DdMgr *defaultDdMgr, *tmpDdMgr;
  Ddi_Dd_t *bdd;

  /*------------ Check for the Correctness of the Command Sequence ----------*/

  if (fsmMgr == NULL) {
    Pdtutil_Warning (1, "Command Out of Sequence.");
    return(1);
  }

  defaultDdMgr = Fsm_MgrReadDdManager (fsmMgr);

  /*----------------------- Take Main and Secondary Name --------------------*/
  
  Pdtutil_ReadSubstring (string, &stringFirst, &stringSecond);

  /*------------------------ Operation on the FSM Manager -------------------*/

  if (stringFirst == NULL) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "FSMManager Verbosity %s (%d)\n",
          Pdtutil_VerbosityEnum2String (Fsm_MgrReadVerbosity (fsmMgr)),
          Fsm_MgrReadVerbosity (fsmMgr));
        return (0);
        break;
      case Pdtutil_MgrOpStats_c:
        Fsm_MgrPrintStats (fsmMgr);
        break;
      case Pdtutil_MgrOpMgrRead_c:
        *voidPointer = (void *) Fsm_MgrReadDdManager (fsmMgr);
        *returnFlagP = Pdtutil_MgrRetDdMgr_c;
        break;
      case Pdtutil_MgrOpMgrSet_c:
        tmpDdMgr = (Ddi_DdMgr *) *voidPointer;
        Fsm_MgrSetDdManager (fsmMgr, tmpDdMgr);
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on FSM Manager");
	break;
    }

    return (0);
  }

  /*----------------------------- Package is DDI ----------------------------*/

  if (strcmp (stringFirst, "ddm")==0) {
    Ddi_MgrOperation (&fsmMgr->dd, stringSecond, operationFlag,
      voidPointer, returnFlagP);

    if (*returnFlagP == Pdtutil_MgrOpMgrDelete_c) {
      fsmMgr->dd = NULL;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*-------------------------------- Options -------------------------------*/

  if (strcmp(stringFirst, "verbosity")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
	Fsm_MgrSetVerbosity (fsmMgr,
	  Pdtutil_VerbosityString2Enum (*voidPointer));
	break;
      case Pdtutil_MgrOpOptionShow_c:
	fprintf (stdout, "Verbosity %s (%d)\n",
          Pdtutil_VerbosityEnum2String (Fsm_MgrReadVerbosity (fsmMgr)),
          Fsm_MgrReadVerbosity (fsmMgr));
	break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on FSM Manager");
	break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*-------------------------- Transition Relation --------------------------*/

  if (strcmp(stringFirst, "tr")==0) {
    Ddi_BddOperation (defaultDdMgr, &(fsmMgr->tr.bdd),
      stringSecond, operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  

  if (strcmp(stringFirst, "reached")==0) {
    Ddi_BddOperation (defaultDdMgr, &(fsmMgr->reached.bdd),
      stringSecond, operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*---------------------------- Initial States -----------------------------*/

  if (strcmp(stringFirst, "init")==0) {
    Ddi_BddOperation (defaultDdMgr, &(fsmMgr->init.bdd),
      stringSecond, operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*------------------------------- From Set --------------------------------*/

  if (strcmp(stringFirst, "from")==0) {
    bdd = Fsm_MgrReadFromBDD (fsmMgr);
    Ddi_BddOperation (defaultDdMgr, &(fsmMgr->from.bdd),
      stringSecond, operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*---------------------------- Lambda Functions ---------------------------*/

  if (strcmp(stringFirst, "lambda")==0) {
    Ddi_BddArrayOperation (defaultDdMgr, &(fsmMgr->lambda.bdd),
      stringSecond, operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*---------------------------- Delta Functions ----------------------------*/

  if (strcmp(stringFirst, "delta")==0) {
    Ddi_BddArrayOperation (defaultDdMgr, &(fsmMgr->delta.bdd),
      stringSecond, operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*------------------------------- No Match --------------------------------*/

  Pdtutil_Warning (1, "Operation on FSM manager not allowed");
  return (1);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char *
Fsm_MgrReadFileName (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->fileName);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char *
Fsm_MgrReadFsmName (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->fsmName);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_DdMgr * 
Fsm_MgrReadDdManager (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->dd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects        []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadY (
  Fsm_Mgr_t *fsmMgr
  )                  
{
  return (fsmMgr->y);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadZ (
  Fsm_Mgr_t *fsmMgr
  )                  
{
  return (fsmMgr->z);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadSX (
  Fsm_Mgr_t *fsmMgr
  )                 
{
  return (fsmMgr->sx);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Fsm_MgrReadReachedBDD (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->reached.bdd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int
Fsm_MgrReadBddFormat (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->bddFormat);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int 
Fsm_MgrReadNI (
  Fsm_Mgr_t *fsmMgr
  )                 
{
  return (fsmMgr->size.i);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int 
Fsm_MgrReadNO (
  Fsm_Mgr_t *fsmMgr
  )                 
{
  return (fsmMgr->size.o);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int 
Fsm_MgrReadNL (
  Fsm_Mgr_t *fsmMgr
  )                 
{
  return (fsmMgr->size.l);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadOrdFileName (
  Fsm_Mgr_t *fsmMgr
  )       
{
  return (fsmMgr->ord.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char ** 
Fsm_MgrReadOrdNodeName (
  Fsm_Mgr_t *fsmMgr
  )       
{
  return (fsmMgr->ord.nodename);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadOrdNodeid (
  Fsm_Mgr_t *fsmMgr
  )         
{
  return (fsmMgr->ord.nodeid);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char ** 
Fsm_MgrReadNameI (
  Fsm_Mgr_t *fsmMgr
  )             
{
  return (fsmMgr->name.i);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char ** 
Fsm_MgrReadNameO (
  Fsm_Mgr_t *fsmMgr
  )             
{
  return (fsmMgr->name.o);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char ** 
Fsm_MgrReadNameNS (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->name.ns);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char ** 
Fsm_MgrReadNamePS (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->name.ps);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadIndexI (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->index.i);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadIndexO (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->index.o);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadIndexNS (
  Fsm_Mgr_t *fsmMgr
  )           
{
  return (fsmMgr->index.ns);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadIndexPS (
  Fsm_Mgr_t *fsmMgr
  )           
{
  return (fsmMgr->index.ps);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadVarI (
  Fsm_Mgr_t *fsmMgr
  )             
{
  return (fsmMgr->var.i);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadVarO (
  Fsm_Mgr_t *fsmMgr
  )             
{
  return (fsmMgr->var.o);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadVarPS (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->var.ps);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t * 
Fsm_MgrReadVarNS (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->var.ns);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadDeltaName (
  Fsm_Mgr_t *fsmMgr
  )         
{
  return (fsmMgr->delta.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_DdArray_t * 
Fsm_MgrReadDeltaBDD (
  Fsm_Mgr_t *fsmMgr
  )          
{
  return (fsmMgr->delta.bdd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadLambdaName (
  Fsm_Mgr_t *fsmMgr
  )        
{
  return (fsmMgr->lambda.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_DdArray_t * 
Fsm_MgrReadLambdaBDD (
  Fsm_Mgr_t *fsmMgr
  )         
{
  return (fsmMgr->lambda.bdd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadTrName (
  Fsm_Mgr_t *fsmMgr
  )            
{
  return (fsmMgr->tr.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t * 
Fsm_MgrReadTrBDD (
  Fsm_Mgr_t *fsmMgr
  )             
{
  return (fsmMgr->tr.bdd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadInitName (
  Fsm_Mgr_t *fsmMgr
  )          
{
  return (fsmMgr->init.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadInitString (
  Fsm_Mgr_t *fsmMgr
  )        
{
  return (fsmMgr->init.string);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadTrString (
  Fsm_Mgr_t *fsmMgr
  )        
{
  return (fsmMgr->tr.string);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadReachedString (
  Fsm_Mgr_t *fsmMgr
  )        
{
  return (fsmMgr->reached.string);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char * 
Fsm_MgrReadFromString (
  Fsm_Mgr_t *fsmMgr
  )        
{
  return (fsmMgr->init.string);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t * 
Fsm_MgrReadInitBDD (
  Fsm_Mgr_t *fsmMgr
  )           
{
  return (fsmMgr->init.bdd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char *
Fsm_MgrReadFromName (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->from.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char *
Fsm_MgrReadReachedName (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->reached.name);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Fsm_MgrReadFromBDD (
  Fsm_Mgr_t *fsmMgr
  )
{
  return (fsmMgr->from.bdd);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

char ** 
Fsm_MgrReadVarnames (
  Fsm_Mgr_t *fsmMgr
  )          
{
  return (fsmMgr->varnames);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadVarauxids (
  Fsm_Mgr_t *fsmMgr
  )          
{
  return (fsmMgr->varauxids);
}

/**Function********************************************************************
  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

int * 
Fsm_MgrReadInvauxids (
  Fsm_Mgr_t *fsmMgr
  )          
{
  return (fsmMgr->invauxids);
}

/**Function********************************************************************
  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

Pdtutil_VerbLevel_e
Fsm_MgrReadVerbosity (
  Fsm_Mgr_t *fsmMgr
  )          
{
  return (fsmMgr->settings.verbosity);
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetFileName (
  Fsm_Mgr_t *fsmMgr, 
  char *fileName
  )
{
  fsmMgr->fileName = Pdtutil_StrDup (fileName);

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetFsmName (
  Fsm_Mgr_t *fsmMgr, 
  char *fsmName
  )
{
  fsmMgr->fsmName = Pdtutil_StrDup (fsmName);

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetDdManager (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_DdMgr *var
  )          
{
  fsmMgr->dd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetY (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )                  
{
  fsmMgr->y = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetZ (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )                  
{
  fsmMgr->z = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetSX (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )                 
{
  fsmMgr->sx = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void Fsm_MgrSetReachedBDD (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_Dd_t *var
  )
{
  fsmMgr->reached.bdd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetBddFormat (
  Fsm_Mgr_t *fsmMgr,
  int var
  )
{
  fsmMgr->bddFormat = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNI (
  Fsm_Mgr_t *fsmMgr, 
  int var
  )                 
{
  fsmMgr->size.i = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNO (
  Fsm_Mgr_t *fsmMgr, 
  int var
  )                 
{
  fsmMgr->size.o = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNL (
  Fsm_Mgr_t *fsmMgr, 
  int var
  )                 
{
  fsmMgr->size.l = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetOrdFileName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )        
{
  fsmMgr->ord.name = var;

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetOrdNodeName (
  Fsm_Mgr_t *fsmMgr, 
  char **var
  )        
{
  fsmMgr->ord.nodename = var;

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetOrdNodeid (
  Fsm_Mgr_t *fsmMgr,
  int *var
  )
{
  fsmMgr->ord.nodeid = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNameI (
  Fsm_Mgr_t *fsmMgr, 
  char **var
  )              
{
  int ni;
  
  if(var != NULL) {
    ni = Fsm_MgrReadNI(fsmMgr);
    fsmMgr->name.i = Pdtutil_StrArrayDup(var, ni);
  } else {
    fsmMgr->name.i = NULL;
  }
  
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNameO (
  Fsm_Mgr_t *fsmMgr, 
  char **var
  )              
{
  int no;
  
  if(var != NULL) {
    no = Fsm_MgrReadNO(fsmMgr);
    fsmMgr->name.o = Pdtutil_StrArrayDup(var, no);
  } else {
    fsmMgr->name.o = NULL;
  }
 
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNameNS (
  Fsm_Mgr_t *fsmMgr, 
  char **var
  )             
{
  int nl;

  if(var != NULL) {
    nl = Fsm_MgrReadNL(fsmMgr);
    fsmMgr->name.ns = Pdtutil_StrArrayDup(var, nl);
  } else {
    fsmMgr->name.ns = NULL;
  }

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetNamePS (
  Fsm_Mgr_t *fsmMgr, 
  char **var
  )             
{
  int nl;

  if(var != NULL) {
    nl = Fsm_MgrReadNL(fsmMgr);
    fsmMgr->name.ps = Pdtutil_StrArrayDup(var, nl);
  } else {
    fsmMgr->name.ps = NULL;
  }

 
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetIndexI (
  Fsm_Mgr_t *fsmMgr, 
  int *var
  )             
{
  fsmMgr->index.i = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetIndexO (
  Fsm_Mgr_t *fsmMgr, 
  int *var
  )             
{
  fsmMgr->index.o = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetIndexNS (
  Fsm_Mgr_t *fsmMgr, 
  int *var
  )            
{
  fsmMgr->index.ns = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetIndexPS (
  Fsm_Mgr_t *fsmMgr, 
  int *var
  )            
{
  fsmMgr->index.ps = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetVarI (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )               
{
  fsmMgr->var.i = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetVarO (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )               
{
  fsmMgr->var.o = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetVarPS (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )              
{
  fsmMgr->var.ps = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetVarNS (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_VarArray_t *var
  )              
{
  fsmMgr->var.ns = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetDeltaName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )          
{
  fsmMgr->delta.name = Pdtutil_StrDup(var);
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetDeltaBDD (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_DdArray_t *var
  )           
{
  fsmMgr->delta.bdd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetLambdaName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )         
{
  fsmMgr->lambda.name = Pdtutil_StrDup(var);
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetLambdaBDD (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_DdArray_t *var
  )          
{
  fsmMgr->lambda.bdd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetTrName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )             
{
  fsmMgr->tr.name = Pdtutil_StrDup(var);
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetTrBDD (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_Dd_t *var
  )              
{
  fsmMgr->tr.bdd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetInitName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )            
{
  fsmMgr->init.name = Pdtutil_StrDup(var);
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetInitString (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )         
{
  fsmMgr->init.string = Pdtutil_StrDup(var);
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetTrString (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )         
{
  fsmMgr->tr.string = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetFromString (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )         
{
  fsmMgr->from.string = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetReachedString (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )         
{
  fsmMgr->init.string = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetInitBDD (
  Fsm_Mgr_t *fsmMgr, 
  Ddi_Dd_t *var
  )            
{
  fsmMgr->init.bdd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetFromName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )             
{
  fsmMgr->from.name = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetReachedName (
  Fsm_Mgr_t *fsmMgr, 
  char *var
  )             
{
  fsmMgr->reached.name = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetFromBDD (
  Fsm_Mgr_t *fsmMgr,
  Ddi_Dd_t *var
  )
{
  fsmMgr->from.bdd = var;
  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetVarnames (
  Fsm_Mgr_t *fsmMgr,
  char **var
  )
{
  fsmMgr->varnames = var;

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void
Fsm_MgrSetVarnamesOne (
  Fsm_Mgr_t *fsmMgr,
  char *var,
  int i
  )
{
  fsmMgr->varnames[i] = var;

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetVarauxids (
  Fsm_Mgr_t *fsmMgr, 
  int *var
  )          
{
  fsmMgr->varauxids = var;

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetInvauxids (
  Fsm_Mgr_t *fsmMgr, 
  int *var
  )          
{
  fsmMgr->invauxids = var;

  return;
}

/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/

void 
Fsm_MgrSetVerbosity (
  Fsm_Mgr_t *fsmMgr, 
  Pdtutil_VerbLevel_e var
  )          
{
  fsmMgr->settings.verbosity = var;

  return;
}

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

