/**CFile***********************************************************************

  FileName    [fsmUtil.c]

  PackageName [fsm]

  Synopsis    [Utility functions to manipulate a FSM]

  Description []

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#include "fsmInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/


/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis    [Given a string it Returns an Integer.]

  Description [It receives a string; to facilitate the user that string
    can be an easy-to-remember predefined code or an integer number
    (interpreted as a string).
    It returns the integer DDDMP value.]

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Fsm_BddFormatString2Int (
  char *string     /* String to Analyze */
)
{
  if (strcmp (string, "default")==0 || strcmp (string, "0")==0) {
  return (DDDMP_MODE_DEFAULT);
  }

  if (strcmp (string, "text")==0 || strcmp (string, "1")==0) {
  return (DDDMP_MODE_TEXT);
  }

  if (strcmp (string, "binary")==0 || strcmp (string, "2")==0) {
  return (DDDMP_MODE_BINARY);
  }

  Pdtutil_Warning (1, "Choice Not Allowed For Verbosity.");
  return (DDDMP_MODE_DEFAULT);
}

/**Function********************************************************************

  Synopsis    [Given an Integer Returns a string]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

char *
Fsm_BddFormatInt2String (
  int intValue
)
{
  switch (intValue) {
    case DDDMP_MODE_DEFAULT:
      return ("default");
      break;
    case DDDMP_MODE_TEXT:
      return ("text");
      break;
    case DDDMP_MODE_BINARY:
      return ("binary");
      break;
    default:
      Pdtutil_Warning (1, "Choice Not Allowed.");
      return ("default");
      break;
    }
}

/**Function********************************************************************

 Synopsis    [Prints Statistics on the FSM Structure]

 SideEffects [none]

 SeeAlso     [Ddi_MgrPrintStats, Tr_MgrPrintStats, Trav_MgrPrintStats]

******************************************************************************/

void
Fsm_MgrPrintStats (
  Fsm_Mgr_t *fsmMgr
)
{
  fprintf (stdout, "Finite State Machine Manager Name %s\n",
    Fsm_MgrReadFsmName (fsmMgr));
  fprintf (stdout, "FSM File Name: %s\n", Fsm_MgrReadFileName (fsmMgr));
  fprintf (stdout, "Number of PI, PO, FF: %d, %d, %d\n", 
    Fsm_MgrReadNI (fsmMgr), Fsm_MgrReadNO (fsmMgr), Fsm_MgrReadNL (fsmMgr));
  fprintf (stdout, "Total CPU Time: %s\n",
    util_print_time (fsmMgr->fsmTime));

  return;
}

/**Function********************************************************************

 Synopsis    [Prints the permutation on file "tmp.prm"]

 SideEffects [none]

 SeeAlso     []

******************************************************************************/

void
Fsm_MgrPrintPrm (
  Fsm_Mgr_t *fsmMgr
)
{
  int i;
  FILE *fp;
  Ddi_DdMgr *dd = Fsm_MgrReadDdManager(fsmMgr);

  fp = Pdtutil_OpenFile(NULL,"tmp.prm","w");

  for (i=0; i<Ddi_ReadSize(dd); i++) {
    (void) fprintf(fp, "%d\n", Ddi_ReadPerm(dd,i));
  }

  Pdtutil_CloseFile (fp);
  return;
}

/**Function********************************************************************

 Synopsis    [Reads constrain]

 SideEffects [none]

 SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Pdtutil_ReadConstrain(
  Fsm_Mgr_t *fsmMgr,
  char *diagfile
)
{
#if 1
  fprintf (stderr, "Not implemented Pdtutil_ReadConstrain\n");
  return NULL;
#else
  FILE *fp;
  Ddi_DdArray_t *Fa;
  Ddi_Dd_t *tmp, *tr, *constrain, *diag, *X, *S, *Y, *Z, *SeY;
  Ddi_Dd_t *one, *zero;
  int i, j, k, var, fixval;
  char *fname;
  Ddi_DdMgr *dd = Fsm_MgrReadDdManager (fsmMgr);
  char neg = 0;
  one = Ddi_MgrReadOne (Fsm_MgrReadDdManager (fsmMgr));

  /* TR constraining by imposing a partial S=Y diagonal */

  if (diagfile[0] == '!') {
    fname = &diagfile[1];
    neg = 1;
  }
  else
    fname = diagfile;

  if (fname[0] == '$') {
    Fa = Ddi_DdArrayLoad (dd, NULL, NULL, Fsm_MgrReadBDDFormat(fsmMgr),
     &fname[1], NULL);
    diag = Ddi_DdArrayFetch (Fa, 0);
    Ddi_DdArrayInsert (Fa, 0, one);
    Ddi_DdArrayFree (Fa);
  }
  else {
   diag = one;
   fp = fopen (fname, "r");
   if (fp != NULL) {
      fprintf (stdout, "reading %s\n", fname);

   while (fscanf (fp, "%d %d", &i, &fixval) > 0) {
     S = Ddi_VarArrayFetch(Fsm_MgrReadVarPS(fsmMgr),i);
     var = S->index - POLI_FSM_SX_OFFSET + POLI_FSM_Z_OFFSET;
     Y = Ddi_VarArrayFetch(Fsm_MgrReadVarNS(fsmMgr),i);

     switch (fixval) {
      case -2:
        SeY = Ddi_DdXor(Y,S);
        break;
      case -1:
        SeY = Ddi_DdXnor(Y,S);
        break;
      case 0:
        SeY = Ddi_DdNot(S);
        break;
      case 1:
        SeY = S;
/* tolto: da rifare     Ddi_Ref(SeY); */
        break;
      case 2:
        SeY = Ddi_DdNot(Y);
        break;
      case 3:
        SeY = Y;
/* tolto: da rifare     Ddi_Ref(SeY); */
        break;
      default:
        fprintf (stderr, "Error : Unknown val in constrain file selection");
      }

     tmp = Ddi_DdAnd(SeY,diag);
     Ddi_DdFree(diag);
     Ddi_DdFree(SeY);
     diag = tmp;
    } /* end-while */
   } /* end-if */
  } /* end-else */

  if (neg)
    return Ddi_Not(diag);
  else
    return diag;

#endif

}

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

