/**CHeaderFile*****************************************************************

  FileName    [tr.h]

  PackageName [Tr]

  Synopsis    [Funtions to manipolate Transition Relation]

  Description [The <b>Tr</b> (<em>Transition Relation</em>) package provides
    a set of functions to manipolate the Transition Relation.<br>
    The transition relation can be built using the monolithic form or
    clustering heuristics.<br>
    Squaring it, transitive closure, is also possible.<br>   
    External procedures included in this package:
    <ul>
    <li>Tr_DdArrayAndExist() 
    <li>Tr_CreateTrv() 
    <li>Tr_DdArraySort() 
    </ul>
    ]

  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#ifndef _TR
#define _TR

/*---------------------------------------------------------------------------*/
/* Nested includes                                                           */
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdarg.h>
#include "ddi.h"
#include "pdtutil.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/* default value for the four weight in the WEIGHTED-SORT method */
#define TRSORT_DEFAULT_W1 6.0
#define TRSORT_DEFAULT_W2 1.0
#define TRSORT_DEFAULT_W3 1.0
#define TRSORT_DEFAULT_W4 2.0
#define TRSORT_DEFAULT_W5 0.0

/* default value for threshold (number of BDD nodes) for clustering */
#define TR_IMAGE_CLUSTER_SIZE 1000

/* squaring methods */
#define TR_SQUARING_FULL 0
#define TR_SQUARING_POWER 1

/*---------------------------------------------------------------------------*/
/* Structure declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/**Enum************************************************************************

  Synopsis    [Type for Transition Relation Sorting Method.]

  Description []

******************************************************************************/

typedef enum
  {
  Tr_SortNone_c,
  Tr_SortDefault_c,
  Tr_SortMinMax_c,
  Tr_SortSize_c,
  Tr_SortWeight_c,
  Tr_SortWeightDac99_c
  }
Tr_Sort_e;

typedef struct Tr_Mgr_s Tr_Mgr_t;

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN Ddi_Dd_t * Tr_BuildClusteredTR(Tr_Mgr_t *trMgr, Ddi_Dd_t *tr, Ddi_VarSet_t *svars, Ddi_VarSet_t *qvars, Ddi_VarSet_t *rvars, int threshold);
EXTERN Ddi_Dd_t * Tr_BuildMonoTR(Tr_Mgr_t *trMgr, Ddi_Dd_t *tr, Ddi_VarSet_t *svars, Ddi_VarSet_t *qvars, Ddi_VarSet_t *rvars);
EXTERN Ddi_Dd_t * Tr_BuildPartTR(Ddi_DdArray_t *delta, Ddi_VarArray_t *y);
EXTERN void Tr_RemoveLambdaLatches(Tr_Mgr_t *trMgr);
EXTERN Ddi_Dd_t * Tr_BuildTransClosure(Tr_Mgr_t *trMgr, Ddi_Dd_t *TR, Ddi_VarArray_t *s, Ddi_VarArray_t *y, Ddi_VarArray_t *z);
EXTERN Tr_Mgr_t * Tr_MgrInit(char *trName, Ddi_DdMgr *dd);
EXTERN void Tr_MgrQuit(Tr_Mgr_t *trMgr);
EXTERN int Tr_MgrPrintStats(Tr_Mgr_t *trMgr);
EXTERN int Tr_MgrOperation(Tr_Mgr_t *trMgr, char *string, Pdtutil_MgrOp_t operationFlag, void **voidPointer, Pdtutil_MgrRet_t *returnFlagP);
EXTERN void Tr_MgrSetDdiMgrDefault(Tr_Mgr_t *trMgr, Ddi_DdMgr *mgr);
EXTERN Ddi_DdMgr * Tr_MgrReadDdiMgrDefault(Tr_Mgr_t *trMgr);
EXTERN Ddi_VarArray_t * Tr_MgrReadI(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetI(Tr_Mgr_t *trMgr, Ddi_VarArray_t *i);
EXTERN Ddi_VarArray_t * Tr_MgrReadPS(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetPS(Tr_Mgr_t *trMgr, Ddi_VarArray_t *ps);
EXTERN Ddi_VarArray_t * Tr_MgrReadNS(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetNS(Tr_Mgr_t *trMgr, Ddi_VarArray_t *ns);
EXTERN Ddi_Dd_t * Tr_MgrReadTr(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetTr(Tr_Mgr_t *trMgr, Ddi_Dd_t *tr);
EXTERN Pdtutil_VerbLevel_e Tr_MgrReadVerbosity(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetVerbosity(Tr_Mgr_t *trMgr, Pdtutil_VerbLevel_e verbosity);
EXTERN Tr_Sort_e Tr_MgrReadSortMethod(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetSortMethod(Tr_Mgr_t *trMgr, Tr_Sort_e sortMethod);
EXTERN int Tr_MgrReadClustThreshold(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetClustThreshold(Tr_Mgr_t *trMgr, int ClustThreshold);
EXTERN double Tr_MgrReadSortW(Tr_Mgr_t *trMgr, int i);
EXTERN void Tr_MgrSetSortW(Tr_Mgr_t *trMgr, int i, double SortW);
EXTERN void Tr_MgrSetTrName(Tr_Mgr_t *trMgr, char *trName);
EXTERN char * Tr_MgrReadTrName(Tr_Mgr_t *trMgr);
EXTERN int Tr_MgrReadMaxIter(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetMaxIter(Tr_Mgr_t *trMgr, int maxIter);
EXTERN int Tr_MgrReadSquaringMethod(Tr_Mgr_t *trMgr);
EXTERN void Tr_MgrSetSquaringMethod(Tr_Mgr_t *trMgr, int method);
EXTERN int Tr_DdSortPartitions(Tr_Mgr_t *TrMgr, Ddi_Dd_t *F, Ddi_VarSet_t *svars, Ddi_VarSet_t *qvars, Ddi_VarSet_t *rvars, Tr_Sort_e method);
EXTERN Tr_Sort_e Tr_TrSortString2Enum(char *string);
EXTERN char * Tr_TrSortEnum2String(Tr_Sort_e enumType);

/**AutomaticEnd***************************************************************/


#endif /* _TR */
