/**CFile***********************************************************************

  FileName    [trav.c]

  PackageName [trav]

  Synopsis    [Main module for a simple traversal of finite state machine]

  Description [External procedure included in this file is:
    <ul>
    <li>TravReadOptions()
    </ul>
    ]

  SeeAlso     []  

  Author      [Gianpiero Cabodi and Stefano Quer]

  Copyright   [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.
    ]
  
  Revision    []

******************************************************************************/

#include "travInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/
  
/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

#define MAX_ROW_LEN 120
#define MAX_OPT_LEN 15

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/


/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis    [Temporary main program to be used as interface for cmd.]

  Description []

  SideEffects [None]

  SeeAlso     []

******************************************************************************/

void
Trav_Main (
  Trav_Mgr_t *travMgr   /* Traversal manager */
)
{
  Ddi_VarSet_t *quantify, *inputs;
  Ddi_Dd_t *reached;
  double nstates;      /* number of reached states */
  Ddi_DdMgr *dd;
  Ddi_DdMgr *ddR;
  Ddi_Dd_t *newTR;  
  long travTime;

  dd = travMgr->ddiMgrTr;
  ddR = travMgr->ddiMgrR;

  /*------------------------- Traversal FSM ------------------------------*/

/*@4*/
  quantify = Ddi_VarArrayToVarSet(dd,Trav_MgrReadPS(travMgr));
  if (Trav_MgrReadSmoothVar (travMgr) == TRAV_SMOOTH_SX) {
    inputs = Ddi_VarArrayToVarSet(dd,Trav_MgrReadI(travMgr));
    quantify = Ddi_VarSetEvalFree(Ddi_VarSetUnion(quantify,inputs),quantify);
    Ddi_VarSetFree(inputs);
  }

  /*------------------------- TR Tracing activation ------------------------*/

  if (Trav_MgrReadEnableTrProfile (travMgr) != 0) {
    Ddi_ProfileInfoInit(Trav_MgrReadTr(travMgr),NULL,(unsigned char)0);
  }

  /*-------------------------   Traversal   ------------------------*/

  /* old reach freed within procedure @@@ controllare !!! */

  travTime = util_cpu_time();

  /* init an empty traversal manager to store traversal settings */

  reached = Trav_Traversal (travMgr,
    Trav_MgrReadTr(travMgr),Trav_MgrReadFrom(travMgr),
    Trav_MgrReadReached(travMgr),
    Trav_MgrReadPS(travMgr),Trav_MgrReadNS(travMgr),quantify);

  Ddi_VarSetFree (quantify);
  
  travTime = util_cpu_time() - travTime;
  travMgr->travTime += travTime;

  if (reached==NULL) {
    Pdtutil_Warning (1, "Traversal is failed.");
    return;
  }

  /*------------------------- TR Tracing activation ------------------------*/

  if (Trav_MgrReadEnableTrProfile (travMgr) != 0) {
    Ddi_ProfileInfoPrint(Trav_MgrReadTr(travMgr));
    if (Trav_MgrReadTrPruneThreshold (travMgr) >= 0) {
      newTR = Ddi_PruneProfiled (
        Trav_MgrReadTr (travMgr),
        Trav_MgrReadEnableTrProfile (travMgr),
        Trav_MgrReadTrPruneThreshold (travMgr));
      fprintf (stdout,
        "Pruned TR size : %d -> %d\n",Ddi_DdSize(Trav_MgrReadTr(travMgr)), 
               Ddi_DdSize(newTR));
      Ddi_ProfileInfoFree(Trav_MgrReadTr(travMgr));
      Ddi_DdFree(Trav_MgrReadTr(travMgr));
      Trav_MgrSetTr(travMgr,newTR);
    }
  } else {
    if (Trav_MgrReadTrPruneThreshold (travMgr) >= 0) {
      newTR = Ddi_DdRestrict(Trav_MgrReadTr(travMgr),
        Trav_MgrReadReached(travMgr));
      fprintf (stdout,
        "Restrict-ed (with R) TR size : %d -> %d\n",
        Ddi_DdSize (Trav_MgrReadTr(travMgr)), Ddi_DdSize (newTR));
      Ddi_DdFree (Trav_MgrReadTr(travMgr));
      Trav_MgrSetTr(travMgr, newTR);
    }
  }

  Pdtutil_ChrPrint (stdout, '*', 50);
  fprintf (stdout, "\n");
  fprintf(stdout, "Reachability Analysis Results Summary:\n");
  fprintf (stdout, "Traversal depth       : %d\n",
    Trav_MgrReadLevel (travMgr) - 1);
  fprintf (stdout, "# REACHED size        : %d\n", Ddi_DdSize(reached));

  if (Trav_MgrReadVerbosity (travMgr) >=   Pdtutil_VerbLevelUsrMed_c) {
    /* Total number of reached states */
    nstates = Ddi_CountMinterm(reached,Ddi_ReadSize(dd)) / 
      pow(2.0,(double) (Ddi_ReadSize(dd) - 
      Ddi_VarArray_n(Trav_MgrReadPS(travMgr))));
    fprintf (stdout, "# REACHED states      : %g\n", nstates);

    fprintf (stdout, "Max. Internal Product : %d\n",
      Trav_MgrReadProductPeak (travMgr));

    fprintf (stdout, "CPU Time              : %s\n",
      util_print_time (travTime));
  }
  Pdtutil_ChrPrint (stdout, '*', 50);
  fprintf (stdout, "\n");

  /*--------------------- Partitioning Reached ----------------------------*/

#if 0
  if (Trav_MgrReadVerbosity (travMgr) > 2) {
    Part_DdDisjSuppPart (reached, Trav_MgrReadTr(travMgr), 
      Trav_MgrReadPS(travMgr), Trav_MgrReadNS(travMgr),
      Trav_MgrReadVerbosity (travMgr));
  }
#endif
  
  /*------------------------------ Exit -----------------------------------*/

  return;
}


/*---------------------------------------------------------------------------*/
/* Definition of static functions                                            */
/*---------------------------------------------------------------------------*/
