/**CHeaderFile*****************************************************************

  FileName    [travInt.h]

  PackageName [trav]

  Synopsis    [Definition of the struct of FSM and struct for options]

  Description []
 
  SeeAlso     []  

  Author    [Gianpiero Cabodi and Stefano Quer]

  Copyright [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#ifndef _TRAVINT
#define _TRAVINT

/*---------------------------------------------------------------------------*/
/* Nested includes                                                           */
/*---------------------------------------------------------------------------*/

#include "dddmp.h"
#include "ddi.h"
#include "tr.h"
#include "part.h"
#include "trav.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Structure declarations                                                     */
/*---------------------------------------------------------------------------*/

/**Struct*********************************************************************
 Synopsis    [Traversal Manager]
 Description [The Traversal manager structure]
******************************************************************************/

struct Trav_Mgr_s {
  char *travName;

  Ddi_DdMgr         *ddiMgrTr;        /* TR manager */
  Ddi_DdMgr         *ddiMgrR;         /* Reached states manager */
  Ddi_DdMgr         *ddiMgrAux;       /* auxiliary manager */
  Ddi_Dd_t          *tr;              /* transition relation */
  Ddi_VarArray_t    *i;               /* primary inputs */
  Ddi_VarArray_t    *ps;              /* present state variables */
  Ddi_VarArray_t    *ns;              /* next state variables */
  Ddi_Dd_t          *from;            /* from set */
  Ddi_Dd_t          *reached;         /* reached set */
  Ddi_Dd_t          *assert;          /* assertion for reached */
  int               level;            /* traversal level */ 
  int               productPeak;      /* Product Peak value */

  long travTime;                      /* Total Traversal Time */

  struct {
    Pdtutil_VerbLevel_e verbosity;
    int logPeriod;                  /* Period for verbosity activation */
    int savePeriod;                 /* Period for saving intermediate BDDs */
    char *savePeriodName;           /* Period Name (see savePeriod) */
    int maxIter;                    /* max allowed traversal iterations */
    Trav_ImgMethod_e imgMethod;     /* selects image method */
    int imgCofactor;                /* enables cofactoring with from */
    int partThFrom;                 /* partitioning threshold for from */
    int partThTr;                   /* partitioning threshold for from */
    int mgrAuxFlag;                 /* Flag to indicate if to use or not
                                       the auxiliary manager during
                                       traversal */
    Trav_FromSelect_e fromSelect;   /* from selection */

    Part_Method_e partitionMethod; 

    /* Old trav_opt options */

    int enableDdR,     /* 1 if second dd manager for reached enabled */
        enableTrProfile,   /* 1 if TR tracing enabled */
        trPruneThreshold,/* -1 if TR pruning disabled */
        removeLLatches,  /* if 1 do lambda latches removal */
        partDisjTrThresh, /* 1= disjoint TR partitioning threshold */
        squaring,      /* 1 enable squaring */
        smoothVars,   /* select variables for smoothing within
                    img/closure */
        Iwls95Variant,  /* variant of IWLS95 image/traversal method */
        sorting;       /* method of sorting ( 0 = no sorting ) */

    int threshold;       /* threshold (size of BDD) for build clusters */

    /* weight for Weighted sorting method */
    double w1,
           w2,
           w3,
           w4;
  
  } settings;
};	

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/


/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/


#endif /* _TRAVINT */
