/**CFile***********************************************************************

  FileName    [travMgr.c]

  PackageName [trav]

  Synopsis    [Functions to handle a Trav manager]

  Description []

  SeeAlso     []  

  Author      [Gianpiero Cabodi and Stefano Quer]

  Copyright   [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

#include "travInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/
  
/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/
  
/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/


/**Function********************************************************************

  Synopsis    [Create a Manager.]

  Description []

  SideEffects [none]

  SeeAlso     [Ddi_DdiMgrQuit]

******************************************************************************/

Trav_Mgr_t *
Trav_MgrInit (
  char *travName   /* Name of the Trav structure */,
  Ddi_DdMgr *dd    /* ddi manager */
)
{
  Trav_Mgr_t *travMgr;

  /*------------------------- Allocation -----------------------------*/

  travMgr = Pdtutil_Alloc (Trav_Mgr_t, 1);
  if (travMgr == NULL) {
    fprintf (stderr, "travMgr.c Error: Out of memory.\n");
    return (NULL);
  }

  /*------------------- Set the Name of the Structure ------------------*/

  if (travName == NULL) {
    travName = "trav";
  }

  travMgr->travName = Pdtutil_StrDup(travName);

  /*------------------------ Initializations -------------------------*/

  travMgr->ddiMgrTr = dd;
  travMgr->ddiMgrR = dd;
  travMgr->ddiMgrAux = NULL;
  travMgr->tr = NULL;
  travMgr->i  = NULL;
  travMgr->ns = NULL;
  travMgr->ps = NULL;
  travMgr->from = NULL;
  travMgr->reached = NULL;
  travMgr->assert = NULL;
  travMgr->level = 1;
  travMgr->productPeak = 0;
  travMgr->travTime = 0.0;

  travMgr->settings.verbosity = Pdtutil_VerbLevelUsrMax_c;
  travMgr->settings.logPeriod = 1;
  travMgr->settings.savePeriod = -1;
  travMgr->settings.savePeriodName = NULL;
  travMgr->settings.maxIter = -1;
  travMgr->settings.imgMethod = Trav_ImgMethodIwls_c;
  travMgr->settings.imgCofactor = 0;
  travMgr->settings.fromSelect = Trav_FromSelectCofactor_c;
  travMgr->settings.partThFrom = -1;
  travMgr->settings.partThTr = -1;
  travMgr->settings.mgrAuxFlag = 0;

  travMgr->settings.partitionMethod = Part_MethodNone_c;

  /*----------------- Old Options Inizializations --------------------*/

  travMgr->settings.squaring = 0;
  travMgr->settings.enableDdR = 0;
  travMgr->settings.enableTrProfile = 0;
  travMgr->settings.trPruneThreshold = -1;
  travMgr->settings.removeLLatches = 0;
  travMgr->settings.smoothVars = TRAV_SMOOTH_SX;
  travMgr->settings.Iwls95Variant = Trav_ImgMethodIwlsClassic_c;
  travMgr->settings.partDisjTrThresh = -1;
  travMgr->settings.sorting = 0;
  travMgr->settings.w1 = TRSORT_DEFAULT_W1;
  travMgr->settings.w2 = TRSORT_DEFAULT_W2;
  travMgr->settings.w3 = TRSORT_DEFAULT_W3;
  travMgr->settings.w4 = TRSORT_DEFAULT_W4;
  travMgr->settings.threshold = TR_IMAGE_CLUSTER_SIZE;

  return (travMgr);
}

/**Function********************************************************************

  Synopsis    [Closes a Manager.]

  Description []

  SideEffects [none]

  SeeAlso     [Ddi_DdiMgrInit]

******************************************************************************/

void
Trav_MgrQuit(
  Trav_Mgr_t *travMgr     /* Traversal manager */
)
{
  Pdtutil_Free(travMgr->travName);
  
  Ddi_VarArrayFree (travMgr->i);
  Ddi_VarArrayFree (travMgr->ps);
  Ddi_VarArrayFree (travMgr->ns);

  Ddi_DdFree (travMgr->tr);
  Ddi_DdFree (travMgr->from);
  Ddi_DdFree (travMgr->reached);

  Ddi_DdFree (travMgr->assert);

  Pdtutil_Free (travMgr->settings.savePeriodName);

  Pdtutil_Free (travMgr);

  return;
}

/**Function********************************************************************

  Synopsis           [Print Statistics on the Traversal Manager.]

  Description        []

  SideEffects        []

  SeeAlso            []

******************************************************************************/

int
Trav_MgrPrintStats (
  Trav_Mgr_t *travMgr     /* Traversal manager */
  )
{
  fprintf (stdout, "Traversal Manager Name %s\n",
    travMgr->travName);
  fprintf (stdout, "Total CPU Time: %s\n",
    util_print_time (travMgr->travTime));
 
  return (1);
}


/**Function********************************************************************

  Synopsis    [Perform an Operation on the Traversal Manager.]

  Description []

  SideEffects []

  SeeAlso     [CmdMgrOperation, CmdRegOperation, Fsm_MgrOperation,
    Tr_MgrOperation]

******************************************************************************/

int
Trav_MgrOperation (
  Trav_Mgr_t *travMgr              /* Traversal Manager */,
  char *string                     /* String */,
  Pdtutil_MgrOp_t operationFlag    /* Operation Flag */,
  void **voidPointer               /* Generic Pointer */,
  Pdtutil_MgrRet_t *returnFlagP    /* Type of the Pointer Returned */
  )
{
  char *stringFirst, *stringSecond;
  Ddi_DdMgr *defaultDdMgr, *tmpDdMgr;

  /*------------ Check for the Correctness of the Command Sequence ----------*/

  if (travMgr == NULL) {
    Pdtutil_Warning (1, "Command Out of Sequence.");
    return(1);
  }

  defaultDdMgr = Trav_MgrReadDdiMgrDefault (travMgr);

  /*----------------------- Take Main and Secondary Name --------------------*/

  Pdtutil_ReadSubstring (string, &stringFirst, &stringSecond);

  /*----------------------- Operation on the TRAV Manager -------------------*/

  if (stringFirst == NULL) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "TraversalManager verbosity %s (%d)\n",
          Pdtutil_VerbosityEnum2String (Trav_MgrReadVerbosity (travMgr)),
          Trav_MgrReadVerbosity (travMgr));
        fprintf (stdout, "TraversalManager logPeriod %d\n",
          Trav_MgrReadLogPeriod (travMgr));
        fprintf (stdout, "TraversalManager savePeriod %d\n",
          Trav_MgrReadSavePeriod (travMgr));
        fprintf (stdout, "TraversalManager savePeriodName %s\n",
          Trav_MgrReadSavePeriodName (travMgr));
        fprintf (stdout, "TraversalManager maxIter %d\n",
          Trav_MgrReadMaxIter (travMgr));
        fprintf (stdout, "TraversalManager imgMethod %s (%d)\n",
          Trav_ImgMethodEnum2String (Trav_MgrReadImgMethod (travMgr)),
          Trav_MgrReadImgMethod (travMgr));
        fprintf (stdout, "TraversalManager imgCofactor %d\n",
          Trav_MgrReadImgCofactor (travMgr));
        fprintf (stdout, "TraversalManager fromSelect %s (%d)\n",
          Trav_FromSelectEnum2String (Trav_MgrReadFromSelect (travMgr)),
          Trav_MgrReadFromSelect (travMgr));
        fprintf (stdout, "TraversalManager partThFrom %d\n",
          Trav_MgrReadPartThFrom (travMgr));
        fprintf (stdout, "TraversalManager partThTR %d\n",
          Trav_MgrReadPartThTr (travMgr));
        break;
      case Pdtutil_MgrOpStats_c:
        Trav_MgrPrintStats(travMgr);
        break;
      case Pdtutil_MgrOpMgrRead_c:
        *voidPointer = (void *) Trav_MgrReadDdiMgrDefault (travMgr);
        *returnFlagP = Pdtutil_MgrRetDdMgr_c;
        break;
      case Pdtutil_MgrOpMgrSet_c:
        tmpDdMgr = (Ddi_DdMgr *) voidPointer;
        Trav_MgrSetDdiMgrDefault (travMgr, tmpDdMgr);
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }
    return (0);
  }

  /*----------------------------- Package is DDI ----------------------------*/

  if ( strcmp (stringFirst, "ddmr")==0 ) {
    Ddi_MgrOperation (&(travMgr->ddiMgrR), stringSecond, operationFlag,
      voidPointer, returnFlagP);

    if (*returnFlagP == Pdtutil_MgrOpMgrDelete_c) {
      travMgr->ddiMgrR = NULL;
    }
 
    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if ( strcmp (stringFirst, "ddmtr")==0 ) {
    Ddi_MgrOperation (&(travMgr->ddiMgrTr), stringSecond, operationFlag,
      voidPointer, returnFlagP);

    if (*returnFlagP == Pdtutil_MgrOpMgrDelete_c) {
      travMgr->ddiMgrTr = NULL;
    }
 
    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*-------------------------------- Options --------------------------------*/

  if (strcmp(stringFirst, "verbosity")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetVerbosity (travMgr,
          Pdtutil_VerbosityString2Enum (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "Verbosity %s (%d)\n",
          Pdtutil_VerbosityEnum2String (Trav_MgrReadVerbosity (travMgr)),
          Trav_MgrReadVerbosity (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "logPeriod")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetLogPeriod (travMgr, atoi (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "LogPeriod %d\n", Trav_MgrReadLogPeriod (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "savePeriod")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetSavePeriod (travMgr, atoi (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "SavePeriod %d\n", Trav_MgrReadSavePeriod (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "savePeriodName")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetSavePeriodName (travMgr, (char *) *voidPointer);
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "SavePeriod %s\n",
          Trav_MgrReadSavePeriodName (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "maxIter")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetMaxIter (travMgr, atoi (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "MaxIter %d\n", Trav_MgrReadMaxIter (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "imgMethod")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetImgMethod (travMgr,
          Trav_ImgMethodString2Enum (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "ImgMethod %s (%d)\n",
          Trav_ImgMethodEnum2String (Trav_MgrReadImgMethod (travMgr)),
          Trav_MgrReadImgMethod (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "imgCofactor")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetImgCofactor (travMgr, atoi (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "ImgCofactor %d\n",
	  Trav_MgrReadImgCofactor (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "fromSelect")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetFromSelect (travMgr,
          Trav_FromSelectString2Enum (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "FromSelect %s (%d)\n",
          Trav_FromSelectEnum2String (Trav_MgrReadFromSelect (travMgr)),
          Trav_MgrReadFromSelect (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "partThFrom")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetPartThFrom (travMgr, atoi (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "PartThFrom %d\n", Trav_MgrReadPartThFrom (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  if (strcmp(stringFirst, "partThTr")==0) {
    switch (operationFlag) {
      case Pdtutil_MgrOpOptionSet_c:
        Trav_MgrSetPartThTr (travMgr, atoi (*voidPointer));
        break;
      case Pdtutil_MgrOpOptionShow_c:
        fprintf (stdout, "PartThTR %d\n", Trav_MgrReadPartThTr (travMgr));
        break;
      default:
        Pdtutil_Warning (1, "Operation Non Allowed on TRAV Manager");
        break;
    }

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*-------------------------- Transition Relation --------------------------*/
  
  if (strcmp(stringFirst, "tr")==0) {
    Ddi_BddOperation (defaultDdMgr, &(travMgr->tr), stringSecond,
      operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*------------------------------ Reached Set ------------------------------*/

  if (strcmp(stringFirst, "reached")==0) {
    Ddi_BddOperation (defaultDdMgr, &(travMgr->reached), stringSecond,
      operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*-------------------------------- From Set -------------------------------*/

  if (strcmp(stringFirst, "from")==0) {
    Ddi_BddOperation (defaultDdMgr, &(travMgr->from), stringSecond,
      operationFlag, voidPointer,
      returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }
  
  /*------------------------------- Assertion -------------------------------*/

  if (strcmp(stringFirst, "assert")==0) {
    Ddi_BddOperation (defaultDdMgr, &(travMgr->assert), stringSecond,
      operationFlag, voidPointer, returnFlagP);

    Pdtutil_Free (stringFirst);
    Pdtutil_Free (stringSecond);
    return (0);
  }

  /*------------------------------- No Match --------------------------------*/

  Pdtutil_Warning (1, "Operation on TRAV manager not allowed");
  return (1);
}


/**Function********************************************************************

  Synopsis    [Read transition relation]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Trav_MgrReadTr(
  Trav_Mgr_t *travMgr            /* traversal manager */
  )
{
  return (travMgr->tr);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetTr (
  Trav_Mgr_t *travMgr       /* Traversal manager */,
  Ddi_Dd_t   *tr            /* transition relation */
)
{
  if (travMgr->tr != NULL) {
    Ddi_DdFree (travMgr->tr);
  }
  travMgr->tr = Ddi_DdCopy (travMgr->ddiMgrTr, tr);

  return;
}

/**Function********************************************************************

  Synopsis    [Read Number of Level]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadLevel (
  Trav_Mgr_t *travMgr            /* traversal manager */
  )
{
  return (travMgr->level);
}

/**Function********************************************************************

  Synopsis    [Read Product Peak Value]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadProductPeak (
  Trav_Mgr_t *travMgr    /* Traversal manager */
  )
{
  return (travMgr->productPeak);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetProductPeak (
  Trav_Mgr_t *travMgr       /* Traversal Manager */,
  int productPeak           /* Product Peak  */
)
{
  travMgr->productPeak = productPeak;

  return;
}

/**Function********************************************************************

  Synopsis    [Read PI array]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t *
Trav_MgrReadI(
  Trav_Mgr_t *travMgr            /* tr manager */
)
{
  return (travMgr->i);
}

/**Function********************************************************************

  Synopsis    [Set the PI array]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetI(
  Trav_Mgr_t *travMgr      /* Traversal manager */,
  Ddi_VarArray_t *i        /* Array of variables */
)
{
  travMgr->i = Ddi_VarArrayCopy(travMgr->ddiMgrTr,i);
}

/**Function********************************************************************

  Synopsis    [Read PS array]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t *
Trav_MgrReadPS(
  Trav_Mgr_t *travMgr    /* Traversal Manager */
)
{
  return (travMgr->ps);
}

/**Function********************************************************************

  Synopsis    [Set the PS array]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetPS(
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  Ddi_VarArray_t *ps       /* Array of Variables */
)
{
  travMgr->ps = Ddi_VarArrayCopy(travMgr->ddiMgrTr,ps);
}

/**Function********************************************************************

  Synopsis    [Set the flag to indicate that during traversal there is an
    additional auziliary manager]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetMgrAuxFlag (
  Trav_Mgr_t *travMgr            /* traversal manager */,
  int flag
)
{
  travMgr->settings.mgrAuxFlag = flag;

  return;
}

/**Function********************************************************************

  Synopsis    [Read NS array]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_VarArray_t *
Trav_MgrReadNS(
  Trav_Mgr_t *travMgr            /* tr manager */
)
{
  return (travMgr->ns);
}

/**Function********************************************************************

  Synopsis    [Set the NS array]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetNS(
  Trav_Mgr_t *travMgr     /* Traversal manager */,
  Ddi_VarArray_t *ns      /* Array of variables */
)
{
  travMgr->ns = Ddi_VarArrayCopy(travMgr->ddiMgrTr,ns);
}

/**Function********************************************************************

  Synopsis    [Read reached]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Trav_MgrReadReached(
  Trav_Mgr_t *travMgr   /* Traversal manager */
)
{
  return (travMgr->reached);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetReached (
  Trav_Mgr_t *travMgr       /* Traversal manager */,
  Ddi_Dd_t *reached         /* Reached set */
)
{
  if (travMgr->reached != NULL)
    Ddi_DdFree(travMgr->reached);
  travMgr->reached = Ddi_DdCopy(travMgr->ddiMgrR,reached);

  return;
}


/**Function********************************************************************

  Synopsis    [Read assert]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Trav_MgrReadAssert(
  Trav_Mgr_t *travMgr    /* Traversal manager */
)
{
  return (travMgr->assert);
}

/**Function********************************************************************

  Synopsis    [Set Assert]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetAssert (
  Trav_Mgr_t *travMgr       /* Traversal manager */,
  Ddi_Dd_t *assert          /* Assertion */
)
{
  if (travMgr->assert != NULL)
    Ddi_DdFree(travMgr->assert);
  travMgr->assert = Ddi_DdCopy(travMgr->ddiMgrR,assert);

  return;
}

/**Function********************************************************************

  Synopsis    [Read from]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_Dd_t *
Trav_MgrReadFrom(
  Trav_Mgr_t *travMgr   /* Traversal manager */
)
{
  return (travMgr->from);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetFrom (
  Trav_Mgr_t *travMgr       /* Traversal manager */,
  Ddi_Dd_t *from            /* From set */
)
{
  if (travMgr->from != NULL) {
    Ddi_DdFree(travMgr->from);
  }
  travMgr->from = Ddi_DdCopy(travMgr->ddiMgrR,from);

  return;
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

char *
Trav_MgrReadName (
  Trav_Mgr_t *travMgr       /* Traversal manager */
)
{
  return (travMgr->travName);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetName (
  Trav_Mgr_t *travMgr       /* Traversal manager */,
  char *travName
)
{
  travMgr->travName = Pdtutil_StrDup (travName);

  return;
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadSmoothVar(
  Trav_Mgr_t *travMgr       /* Traversal manager */
)
{
  return (travMgr->settings.smoothVars);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

double
Trav_MgrReadW1 (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.w1);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

double
Trav_MgrReadW2 (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.w2);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

double
Trav_MgrReadW3 (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.w3);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

double
Trav_MgrReadW4 (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.w4);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadEnableDdR(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.enableDdR);
}


/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadEnableTrProfile (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.enableTrProfile);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadTrPruneThreshold (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.trPruneThreshold);
}


/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadThreshold (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.threshold);
}


/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadPaDiTrTh (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.partDisjTrThresh);
}


/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Part_Method_e
Trav_MgrReadPartitionMethod (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.partitionMethod);
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetPartitionMethod (
  Trav_Mgr_t *travMgr              /* Traversal Manager */,
  Part_Method_e partitionMethod    /* Partition Method */
)
{
  travMgr->settings.partitionMethod = partitionMethod;
}

/**Function********************************************************************

  Synopsis    [Read ]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadSquaring (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.squaring);
}


/**Function********************************************************************

  Synopsis    [Read verbosity]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Pdtutil_VerbLevel_e
Trav_MgrReadVerbosity(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.verbosity);
}

/**Function********************************************************************

  Synopsis    [Read verbosity]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetSorting (
  Trav_Mgr_t *travMgr       /* Traversal Manager */,
  int sorting
)
{
  travMgr->settings.sorting = sorting;

  return;
}


/**Function********************************************************************

  Synopsis    [Read verbosity]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadSorting (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.sorting);
}

/**Function********************************************************************

  Synopsis    [Read the maximum number of traversal iterations]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadMaxIter(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.maxIter);
}


/**Function********************************************************************

  Synopsis    [Read the period for verbosity enabling]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadLogPeriod(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.logPeriod);
}



/**Function********************************************************************

  Synopsis    [Read the period for save BDDs enabling]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadSavePeriod (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.savePeriod);
}



/**Function********************************************************************

  Synopsis    [Read the period for save BDDs enabling]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

char *
Trav_MgrReadSavePeriodName (
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.savePeriodName);
}



/**Function********************************************************************

  Synopsis    [Read the image method selection]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Trav_ImgMethod_e
Trav_MgrReadImgMethod(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.imgMethod);
}


/**Function********************************************************************

  Synopsis    [Read the image flag enabling cofactor with from]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadImgCofactor(
  Trav_Mgr_t *travMgr        /* Traversal Manager */
)
{
  return (travMgr->settings.imgCofactor);
}

/**Function********************************************************************

  Synopsis    [Read from selection]

  SideEffects [none]

  Description []

  SeeAlso     []

******************************************************************************/

Trav_FromSelect_e
Trav_MgrReadFromSelect(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.fromSelect);
}


/**Function********************************************************************

  Synopsis    [Read partitioning threshold for from]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadPartThFrom(
  Trav_Mgr_t *travMgr       /* Traversal Manager */
)
{
  return (travMgr->settings.partThFrom);
}


/**Function********************************************************************

  Synopsis    [Read partitioning threshold for TR]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int
Trav_MgrReadPartThTr(
  Trav_Mgr_t *travMgr         /* Traversal Manager */
)
{
  return (travMgr->settings.partThTr);
}

/**Function********************************************************************

  Synopsis    [Set DDi Mgr TR]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetDdiMgrTr (
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  Ddi_DdMgr *mgrTr         /* dd Manager */
)
{
  travMgr->ddiMgrTr = mgrTr;

  return;
}

/**Function********************************************************************

  Synopsis    [Set default DDi Mgr]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetDdiMgrDefault (
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  Ddi_DdMgr *mgrTr         /* dd Manager */
)
{
  travMgr->ddiMgrTr = mgrTr;

  return;
}

/**Function********************************************************************

  Synopsis    [Set DDi Mgr R]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetDdiMgrR (
  Trav_Mgr_t *travMgr    /* Traversal Manager */,
  Ddi_DdMgr *mgrR        /* Decision Diagram Manager */
)
{
  travMgr->ddiMgrR = mgrR;

  return;
}

/**Function********************************************************************

  Synopsis    [Read default DDi Mgr]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_DdMgr *
Trav_MgrReadDdiMgrDefault (
  Trav_Mgr_t *travMgr      /* Traversal Manager */
)
{
  return (travMgr->ddiMgrTr);
}


/**Function********************************************************************

  Synopsis    [Read DDi Mgr TR]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_DdMgr *
Trav_MgrReadDdiMgrTr (
  Trav_Mgr_t *travMgr      /* Traversal Manager */
)
{
  return (travMgr->ddiMgrTr);
}

/**Function********************************************************************

  Synopsis    [Read DDi Mgr R]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Ddi_DdMgr *
Trav_MgrReadDdiMgrR (
  Trav_Mgr_t *travMgr      /* Traversal Manager */
)
{
  return (travMgr->ddiMgrR);
}

/**Function********************************************************************

  Synopsis    [Set verbosity]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetVerbosity(
  Trav_Mgr_t *travMgr             /* Traversal Manager */,
  Pdtutil_VerbLevel_e verbosity   /* Verbosity Level */
)
{
  travMgr->settings.verbosity = verbosity;

  return;
}

/**Function********************************************************************

  Synopsis    [Set the maximum number of traversal iterations]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetMaxIter(
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  int maxIter              /* Max iterations */
)
{
  travMgr->settings.maxIter = maxIter;
}


/**Function********************************************************************

  Synopsis    [Set the period for verbosity enabling]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetLogPeriod(
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  int logPeriod            /* Period */
)
{
  travMgr->settings.logPeriod = logPeriod;
}


/**Function********************************************************************

  Synopsis    [Set the period for verbosity enabling]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetSavePeriod(
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  int savePeriod           /* Period */
)
{
  travMgr->settings.savePeriod = savePeriod;

  return;
}



/**Function********************************************************************

  Synopsis    [Set the period for verbosity enabling]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetSavePeriodName (
  Trav_Mgr_t *travMgr      /* Traversal Manager */,
  char *savePeriodName     /* Period File Name*/
)
{
  travMgr->settings.savePeriodName = Pdtutil_StrDup (savePeriodName);

  return;
}



/**Function********************************************************************

  Synopsis    [Set the image method selection]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetImgMethod(
  Trav_Mgr_t *travMgr           /* Traversal Manager */,
  Trav_ImgMethod_e imgMethod    /* Image Method */
)
{
  travMgr->settings.imgMethod = imgMethod;

  return;
}

/**Function********************************************************************

  Synopsis    [Sets the image flag enabling cofactor with from]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetImgCofactor(
  Trav_Mgr_t *travMgr       /* Traversal Manager */,
  int imgCofactor           /* Image Cofactor value */
)
{
  travMgr->settings.imgCofactor = imgCofactor;

  return;
}

/**Function********************************************************************

  Synopsis    [Set the from selection]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetFromSelect(
  Trav_Mgr_t *travMgr             /* Traversal Manager */,
  Trav_FromSelect_e fromSelect    /* Selection */
)
{
  if (fromSelect != Trav_FromSelectSame_c) {
    travMgr->settings.fromSelect = fromSelect;
  }

  return;
}

/**Function********************************************************************

  Synopsis    [Set partitioning threshold for from]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetPartThFrom(
  Trav_Mgr_t *travMgr       /* Traversal Manager */,
  int       threshold       /* Threshold value (-1 for no threshold) */
)
{
  travMgr->settings.partThFrom = threshold;
  return;
}

/**Function********************************************************************

  Synopsis    [Set partitioning threshold for TR]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

void
Trav_MgrSetPartThTr(
  Trav_Mgr_t *travMgr     /* Traversal Manager */,
  int threshold           /* Threshold value (-1 for no threshold) */
)
{
  travMgr->settings.partThTr = threshold;
  return;
}

/**Function********************************************************************

  Synopsis    [Read the flag to indicate that during traversal there is an
    additional auziliary manager]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

int 
Trav_MgrReadMgrAuxFlag (
  Trav_Mgr_t *travMgr      /* Traversal Manager */
  )
{
  return (travMgr->settings.mgrAuxFlag);
}




