/**CFile***********************************************************************

  FileName    [travUtil.c]

  PackageName [trav]

  Synopsis    [Utility Functions for the Traverse Package]

  Description []

  SeeAlso     []  

  Author      [Gianpiero Cabodi and Stefano Quer]

  Copyright   [This file was created at the Politecnico di Torino,
    Torino, Italy. 
    The  Politecnico di Torino makes no warranty about the suitability of 
    this software for any purpose.  
    It is presented on an AS IS basis.]
  
  Revision    []

******************************************************************************/

/*---------------------------------------------------------------------------*/
/* Nested includes                                                           */
/*---------------------------------------------------------------------------*/

#include "travInt.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Stucture declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/

/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Static function prototypes                                                */
/*---------------------------------------------------------------------------*/


/**AutomaticEnd***************************************************************/


/*---------------------------------------------------------------------------*/
/* Definition of exported functions                                          */
/*---------------------------------------------------------------------------*/

/**Function********************************************************************

  Synopsis    [Given a string it Returns an Enumerated type]

  Description [It receives a string; to facilitate the user that string
    can be an easy-to-remember predefined code or an integer number
    (interpreted as a string).
    It returns the enumerated type.]

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Trav_FromSelect_e
Trav_FromSelectString2Enum (
  char *string     /* String to Analyze */
  )
{
  if (strcmp (string, "new")==0 || strcmp (string, "0")==0) {
    return (Trav_FromSelectNew_c);
  }

  if (strcmp (string, "reached")==0 || strcmp (string, "1")==0) {
    return (Trav_FromSelectReached_c);
  }

  if (strcmp (string, "to")==0 || strcmp (string, "2")==0) {
    return (Trav_FromSelectTo_c);
  }

  if (strcmp (string, "cofactor")==0 || strcmp (string, "3")==0) {
    return (Trav_FromSelectCofactor_c);
  }

  if (strcmp (string, "restrict")==0 || strcmp (string, "4")==0) {
    return (Trav_FromSelectRestrict_c);
  }

  if (strcmp (string, "best")==0 || strcmp (string, "5")==0) {
    return (Trav_FromSelectBest_c);
  }

  if (strcmp (string, "same")==0 || strcmp (string, "6")==0) {
    return (Trav_FromSelectSame_c);
  }

  Pdtutil_Warning (1, "Choice Not Allowed For From Selection.");
  return (Trav_FromSelectBest_c);
}

/**Function********************************************************************

 Synopsis    [Given an Enumerated type Returns a string]

 Description []

 SideEffects [none]

 SeeAlso     []

******************************************************************************/

char *
Trav_FromSelectEnum2String (
  Trav_FromSelect_e enumType
  )
{
  switch (enumType) {
    case Trav_FromSelectNew_c: 
      return ("new");
      break;
    case Trav_FromSelectReached_c: 
      return ("reached");
      break;
    case Trav_FromSelectTo_c: 
      return ("to");
      break;
    case Trav_FromSelectCofactor_c: 
      return ("cofactor");
      break;
    case Trav_FromSelectRestrict_c: 
      return ("restrict");
      break;
    case Trav_FromSelectBest_c:
      return ("best");
      break;
    case Trav_FromSelectSame_c:
      return ("same");
      break;
    default:
      Pdtutil_Warning (1, "Choice Not Allowed.");
      return ("none");
      break;
  }
}



/**Function********************************************************************

  Synopsis    [Given a string it Returns an Enumerated type]

  Description [It receives a string; to facilitate the user that string
    can be an easy-to-remember predefined code or an integer number
    (interpreted as a string).
    It returns the enumerated type.]

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

Trav_ImgMethod_e
Trav_ImgMethodString2Enum (
  char *string     /* String to Analyze */
  )
{
  if (strcmp (string, "same")==0 || strcmp (string, "0")==0) {
    return (Trav_ImgMethodSame_c);
  }

  if (strcmp (string, "none")==0 || strcmp (string, "1")==0) {
    return (Trav_ImgMethodNone_c);
  }

  if (strcmp (string, "monolithic")==0 || strcmp (string, "2")==0) {
    return (Trav_ImgMethodMonolithic_c);
  }

  if (strcmp (string, "iwls")==0 || strcmp (string, "3")==0) {
    return (Trav_ImgMethodIwls_c);
  }

  if (strcmp (string, "approx")==0 || strcmp (string, "4")==0) {
    return (Trav_ImgMethodApprox_c);
  }

  if (strcmp (string, "iwlsClassic")==0 || strcmp (string, "5")==0) {
    return (Trav_ImgMethodIwlsClassic_c);
  }

  if (strcmp (string, "iwlsPdtrav")==0 || strcmp (string, "6")==0) {
    return (Trav_ImgMethodIwlsPdtrav_c);
  }

  if (strcmp (string, "iwlsConstraint")==0 || strcmp (string, "7")==0) {
    return (Trav_ImgMethodIwlsConstraint_c);
  }

  if (strcmp (string, "iwlsRestrict")==0 || strcmp (string, "8")==0) {
    return (Trav_ImgMethodIwlsRestrict_c);
  }

  Pdtutil_Warning (1, "Choice Not Allowed For From Selection.");
  return (Trav_ImgMethodSame_c);
}

/**Function********************************************************************

 Synopsis    [Given an Enumerated type Returns a string]

 Description []

 SideEffects [none]

 SeeAlso     []

******************************************************************************/

char *
Trav_ImgMethodEnum2String (
  Trav_ImgMethod_e enumType
  )
{
  switch (enumType) {
    case Trav_ImgMethodSame_c: 
      return ("same");
      break;
    case Trav_ImgMethodNone_c:
      return ("none");
      break;
    case Trav_ImgMethodMonolithic_c:
      return ("monolothic");
      break;
    case Trav_ImgMethodIwls_c:
      return ("iwls");
      break;
    case Trav_ImgMethodApprox_c:
      return ("approx");
      break;
    case Trav_ImgMethodIwlsClassic_c:
      return ("iwlsClassic");
      break;
    case Trav_ImgMethodIwlsPdtrav_c:
      return ("iwlsPdtrav");
      break;
    case Trav_ImgMethodIwlsConstraint_c:
      return ("iwlsConstraint");
      break;
    case Trav_ImgMethodIwlsRestrict_c:
      return ("iwls");
      break;
    default:
      Pdtutil_Warning (1, "Choice Not Allowed.");
      return ("none");
      break;
  }
}

/*---------------------------------------------------------------------------*/
/* Definition of internal functions                                          */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Definition of static functions                                            */
/*---------------------------------------------------------------------------*/


